unit DemoDlp;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ShellApi, ExtCtrls, StdCtrls, uCalcDlp; // Run Setup.Bat to copy over uCalc files.

type
  TForm1 = class(TForm)
    txtSumMax: TEdit;
    txtSumExpression: TEdit;
    txtPlotEq: TEdit;
    txtSumResult: TEdit;
    txtResult: TEdit;
    btnEval: TButton;
    btnDefine: TButton;
    btnSum: TButton;
    btnPlot: TButton;
    Label4: TLabel;
    Image1: TImage;
    lblElapsed: TLabel;
    lblFPU: TLabel;
    chkInvalidOp: TCheckBox;
    chkDivisionBy0: TCheckBox;
    chkOverflow: TCheckBox;
    chkUnderflow: TCheckBox;
    chkPrecisionLoss: TCheckBox;
    cmbExpression: TComboBox;
    cmbDefinition: TComboBox;
    cmbNumericFormat: TComboBox;
    procedure btnSumClick(Sender: TObject);
    procedure btnEvalClick(Sender: TObject);
    procedure btnDefineClick(Sender: TObject);
    procedure btnPlotClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure chkInvalidOpClick(Sender: TObject);
    procedure chkDenormalOpClick(Sender: TObject);
    procedure chkDivisionBy0Click(Sender: TObject);
    procedure chkOverflowClick(Sender: TObject);
    procedure chkUnderflowClick(Sender: TObject);
    procedure chkPrecisionLossClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;
  H, W: Variant; { Image1 Height and Width }


implementation

{$R *.DFM}

procedure CartesianLineTo(a: Variant; b: Variant);
var
  x, y: Integer;
begin
  x := W * (a + 10) / 20;
  y := H * (2 - b) / 4;
  Form1.Image1.Canvas.LineTo(x, y);
end;


// uCalc callback routines

// IMPORTANT: Always remember to use  stdcall  for uCalc callbacks
var DisplayAgain: Bool = True;
procedure MyErrorHandler(uCalcHandle: Pointer); stdcall;
var
   ucInst: uCalc;
begin
  ucInst := uCalc.Create(uCalcHandle);
  if DisplayAgain then
  begin
      if MessageDlg('This is a uCalc error message' + Chr(13)+Chr(13)
         + 'Error Handler message: ' + ucInst.ErrorMessage() + Chr(13)
         + 'Offending symbol: ' + ucInst.ErrorSymbol() + Chr(13)
         + 'Error Location: ' + IntToStr(ucInst.ErrorLocation()) + Chr(13) + Chr(13)
         + 'Show this message again for the next error?',
         mtError, [mbYes, mbNo], 0, mbYes) = mrNo then DisplayAgain := False;
   end;

end;

// IMPORTANT: Always remember to use  stdcall  for uCalc callbacks
procedure MyArea(ExprPartPtr: Pointer); stdcall;
var
   MyLength, MyWidth: Extended;
   ep: uCalc.Callback;
begin
   ep := uCalc.Callback.Create(ExprPartPtr);
   MyLength := ep.Arg(1);
   MyWidth := ep.Arg(2);
   
   if MyLength < 0 then ep.ErrorRaiseMessage('Length cannot be negative');
   if MyWidth  < 0 then ep.ErrorRaiseMessage('Width cannot be negative');

   ep.Return(MyLength * MyWidth);
end;

// IMPORTANT: Always remember to use  stdcall  for uCalc callbacks
function NonNativeFunc(a: Double; var b: Longint; c: Byte): Extended; stdcall;
begin
   NonNativeFunc := a + b + c;
end;

// IMPORTANT: Always remember to use  stdcall  for uCalc callbacks
procedure MyIIF_Numeric(ExprPartPtr: Pointer); stdcall;
var
   Condition: Extended;
   TruePart, FalsePart: uCalc.ParsedExpr;
   ep: uCalc.Callback;
begin
   ep := uCalc.Callback.Create(ExprPartPtr);
   Condition := ep.Arg(1);
   TruePart  := ep.ArgExpr(2);
   FalsePart := ep.ArgExpr(3);

   if Condition <> 0
   then ep.Return(TruePart.Evaluate())
   else ep.Return(FalsePart.Evaluate());
end;

// IMPORTANT: Always remember to use  stdcall  for uCalc callbacks
procedure MyIIF_String(ExprPartPtr: Pointer); stdcall;
var
   Condition: Extended;
   TruePart, FalsePart: uCalc.ParsedExpr;
   ep: uCalc.Callback;
begin
  ep := uCalc.Callback.Create(ExprPartPtr);
   Condition := ep.Arg(1);
   TruePart  := ep.ArgExpr(2);
   FalsePart := ep.ArgExpr(3);

   if Condition <> 0
   then ep.ReturnStr(TruePart.EvaluateStr())
   else ep.ReturnStr(FalsePart.EvaluateStr());
end;

// IMPORTANT: Always remember to use  stdcall  for uCalc callbacks
{+++
procedure Native_MsgBox(ExprPartPtr: Pointer); stdcall;
begin
   ucReturnLng(Expr, MessageBox(0, PChar(string(ucArgStr(Expr, 1))), PChar(string(ucArgStr(Expr, 2))), ucArgLng(Expr, 3)));
end;

// IMPORTANT: Always remember to use  stdcall  for uCalc callbacks
function NonNative_MsgBox( Prompt: PAnsiChar; var Title: PAnsiChar; Buttons: Longint): Longint; stdcall;
begin
   NonNative_MsgBox := MessageBox(0, PChar(string(Prompt)), PChar(string(Title)), Buttons);
end;
}

// IMPORTANT: Always remember to use  stdcall  for uCalc callbacks
procedure MyAverage(ExprPartPtr: Pointer); stdcall;
var
   x: Longint;
   Total: Extended;
   ep: uCalc.Callback;
begin
   ep := uCalc.Callback.Create(ExprPartPtr);
   Total := 0;
   for x := 1 to ep.ArgCount() do Total := Total + ep.Arg(x);
   ep.Return(Total / ep.ArgCount());
end;

// IMPORTANT: Always remember to use  stdcall  for uCalc callbacks
procedure MyLeft(ExprPartPtr: Pointer); stdcall;
var
   ep: uCalc.Callback;
begin
  ep := uCalc.Callback.Create(ExprPartPtr);
   ep.ReturnStr(copy(ep.ArgStr(1), 1, Trunc(ep.Arg(2))));
end;

// IMPORTANT: Always remember to use  stdcall  for uCalc callbacks
procedure StringRepeat(ExprPartPtr: Pointer); stdcall;
var
   x: Longint;
   TotalString: AnsiString;
   ep: uCalc.Callback;
begin
   TotalString := '';
   ep := uCalc.Callback.Create(ExprPartPtr);
   for x := 1 To Trunc(ep.Arg(2))
   do TotalString := TotalString + ep.ArgStr(1);
   ep.ReturnStr(TotalString);
end;

procedure MyNumericFormat(ExprPartPtr: Pointer); stdcall;
var
   fmt: string;
   Value: Extended;
   ep: uCalc.Callback;
begin
ep := uCalc.Callback.Create(ExprPartPtr);
   Value := StrToFloat(ep.ArgStr(1));
   fmt := '%g';
   if Form1.cmbNumericFormat.Text = 'Scientific' then fmt := '%e';
   if Form1.cmbNumericFormat.Text = 'Fixed'      then fmt := '%f';
   if Form1.cmbNumericFormat.Text = 'Number'     then fmt := '%n';
   if Form1.cmbNumericFormat.Text = 'Money'      then fmt := '%m';

   ep.ReturnStr(Format(fmt, [Value]));
end;

// IMPORTANT: Always remember to use  stdcall  for uCalc callbacks
procedure ucSum(ExprPartPtr: Pointer); stdcall;
var
   Expression: uCalc.ParsedExpr;
   Variable: uCalc.Item;
   Start, Finish, sStep: Extended;
   x, Total: Extended;
   ep: uCalc.Callback;
begin
ep := uCalc.Callback.Create(ExprPartPtr);
   Expression := ep.ArgExpr(1);
   Start      := ep.Arg(2);
   Finish     := ep.Arg(3);
   sStep      := ep.Arg(4);
   Variable  := ep.ArgObj(5);

   x := Start;
   Total := 0;
   while x <= Finish do
   begin
      Variable.SetVariableValue(x);
      Total := Total + Expression.Evaluate();
      x := x + sStep;
   end;

   ep.Return(Total);
end;

// IMPORTANT: Always remember to use  stdcall  for uCalc callbacks
procedure ucSolve(ExprPartPtr: Pointer); stdcall;
var
   Expression: uCalc.ParsedExpr;
   Variable: uCalc.Item;
   Iterations: Longword;
   a, b, fa, fb, Value, tmp: Double;
   ep: uCalc.Callback;
begin
ep := uCalc.Callback.Create(ExprPartPtr);
   Expression := ep.ArgExpr(1);
   a := ep.Arg(2);
   b := ep.Arg(3);
   Variable := ep.ArgObj(4);
   Iterations := 0;

   Variable.SetVariableValue(a); fa := Expression.Evaluate();
   Variable.SetVariableValue(b); fb := Expression.Evaluate();

   if fb < fa then
   begin  // swap a, b
      tmp := a;
      a := b;
      b := tmp;
   end;

   while Abs(b - a) > 0.000000000000001 do
   begin
      Variable.SetVariableValue((a + b) / 2);

      Value := Expression.Evaluate();

      if Value = 0 then begin a := (a + b) / 2; Break; end;

      if Value < 0
      then a := (a + b) / 2
      else b := (a + b) / 2;

      Iterations := Iterations + 1;
      if Iterations = 100 then Break;
   end;

   If Abs(Value) > 0.0000000001 Then ep.ErrorRaiseMessage('Solution not found');

   ep.Return(a);
end;

{ +++
procedure Test(Expr, Answer: string; t: Longword = 0);
begin
   if Answer <> ucEvalStr(Expr, t) then Application.MessageBox(PChar(Expr + Chr(10) + ucEvalStr(Expr, t) + Chr(10) + Answer), '', 0);
end;

procedure TestExpand(Expr, Answer: string; t: Longword = 0);
begin
   if Answer <> ucExpand(Expr, t) then Application.MessageBox(PChar(Expr + Chr(10) + ucExpand(Expr, t) + Chr(10) + Answer), '', 0);
end;
}

procedure TForm1.FormCreate(Sender: TObject);
begin
   Image1.Canvas.Rectangle(0, 0, Image1.Width, Image1.Height);

   // The definitions here are for "callback" routines.
   // Each line ends with the address of a routine, with the name of the routine preceded by @.
   // The actual code for these routines can be found elsewhere in this file.
   // Do a search for MyErrorHandler, for instance, to see the actual code for it.

   // The MyErrorHandler() function will be called whenever and wherever an
   // error (such as "Syntax Error") is raised.
   uc.AddErrorHandler(@MyErrorHandler);

   // Lets user configure numeric output: Scientific, Fixed, Money, Standard, etc.
   uc.SetOutput(@MyNumericFormat, 'Double');

   // Native callback definitions, which start with "Native: " are recommended for
   // speed, and for routines that use uCalc's default numeric and string types.
   // MyArea() returns the product of multiplying the two arguments.
   // MyLeft() returns the left-most characters of a string.  The second argument represents the number of characters.
   // MsgBox() displays a message box.  The 1st argument is required.  The others are optional.
   uc.DefineFunction('MyArea(Length, Width)', @MyArea);
   uc.DefineFunction('MyLeft(Text As String, Count) As String', @MyLeft);
   //+++uc.DefineFunction('MyMsgBox(Prompt As String, Title As String = "uCalc", Buttons = 0) As Long', @Native_MsgBox);
   uc.DefineSyntax('MsgBox ::= MyMsgBox');

   // The three consecutive dots "..." mean that MyAverage can take any number of arguments
   // (however, because "x" is specified, there must be at least one argument).
   uc.DefineFunction('MyAverage(x ...)', @MyAverage);

   // Two versions of MyIIF can co-exist peacefully with the same name because
   // they are defined with different argument types (numeric vs string).
   // By passing the last two arguments ByExpr, the callback can chose to evaluate only one of of them.
   uc.DefineFunction('MyIIf(ByVal cond, ByExpr TruePart, ByExpr FalsePart)', @MyIIF_Numeric);
   uc.DefineFunction('MyIIf(ByVal cond, ByExpr TruePart As String, ByExpr FalsePart As String) As String', @MyIIF_String);

   // The following defines the "*" operator so that MyString * n returns MyString repeated n times.
   // For instance, "He " * 3, would return "He He He".  20 represents the precedence level.  It is
   // arbitrarily set at the same level as that of the "*" multiplication operator defined in uCalcDlp.Pas.
   uc.DefineOperator('{MyString As String} * {Number} As String', uCalc.Item.Create(uc.HandleOf('*')).Precedence, GroupingEnum.GroupLeftToRight , @StringRepeat);

   // Non-native callbacks are intended for routines that were not created with
   // uCalc in mind, such as the Windows API, or other pre-existing DLL routines.
   // It is convenient (no need to adapt the routines; all you need is a function
   // address), but not as fast as Native callbacks.
   // NonNativeFunc() simply adds the three arguments.
   // nn_MsgBox is a Non-Native version of MsgBox, which displays a message box.
   {+++uc.DefineFunction('NonNativeFunc(ByVal a As Double, ByRef b As Long, ByVal c As Byte) As Extended', @NonNativeFunc);
   uc.DefineFunction('NonNativeMsgBox(ByVal Prompt As String, ByRef Title As String = "uCalc", ByVal Buttons As Long = 0) As Long', @NonNative_MsgBox);
   }
   // This definition is for a summation.  See the callback ucSum() routine for the
   // actual code, which runs a loop that adds up the total for the expression in
   // the first argument a number of times based on the second and third arguments.
   // For instance Sum(g^2+1, 1, 5, 1, g) returns 60 and Sum(x^2, 1, 10) returns 385.
   // The last two arguments are optional, so they default to 1 and x if omitted.
   //
   // The first argument is passed "ByExpr".  So instead of being evaluated before
   // being passed to the callback the way an ordinary argument would be, a handle
   // for the expression is passed so that the callback can evaluate it (in this
   // case numerous times).
   //
   // The last argument is passed "ByHandle".  This causes the callback to receive
   // a handle for the variable being passed, in such a way that it can be linked to
   // the summation counter, and integrated into the expression in the first argument.
   //
   // The actual function being defined is Sum_().  Then a syntax construct named
   // Sum() is defined in such a way that the last argument gets defined as a local
   // variable.  So if you evaluate Sum(x^2, 5, 10, 1, x), the local "x" in this
   // expression will not interfere with a pre-existing variable named x.  Also you
   // do not need to declare a variable ahead of time to use it as a counter for Sum().
   uc.DefineFunction('Sum_(ByExpr Expr, Start, Finish, Step, ByHandle Var)', @ucSum);
   uc.DefineSyntax('Sum({Expr}, {Start}, {Finish} [, {Step=1} [, {Var=x}]])'
                + '::= Local({Var}, Sum_({Expr}, {Start}, {Finish}, {Step}, {Var}))');

   // The following routine solves an equation.
   // The callback code is based on the Bisection Method algorithm.
   // The concept here is very similar to that of Sum().
   // Two syntax constructs are defined.  The second one rearranges the equation
   // if it includes an equal sign.
   // For instance Solve(x^2 = 9+x) becomes Solve(x^2 - (9+x))
   // Solve(x^2 + 1 = 26) returns 5.
   // Solve(x^2 + 1 = 26, -1000, 0) returns -5.
   uc.DefineFunction('Solve_(ByExpr Expr, a, b, ByHandle Var)', @ucSolve);
   uc.DefineSyntax('Solve({Expr} [, {a=-100000000} [, {b=100000000} [, {Var=x}]]]) '
                + '::= Local({Var}, Solve_({Expr}, {a}, {b}, {Var}))');
   uc.DefineSyntax('Solve({Left} = {Right} [, {etc}]) ::= Solve({Left}-({Right}) {etc: , {etc}})');
end;

procedure TForm1.btnSumClick(Sender: TObject);
var
   Expr: uCalc.ParsedExpr;
   VariableX: uCalc.Item;
   x, SumMax, SumTotal: Double;
   Code: Integer;
   TimerStart: TDateTime;
   Elapsed: TTimeStamp;
begin
   x := 1;
   SumTotal := 0;
   VariableX := uc.DefineVariable('x');
   Expr := uc.Parse(txtSumExpression.Text);

   val(txtSumMax.Text, SumMax, Code);
   TimerStart := Time;

   while x <= SumMax do
   begin
     SumTotal := SumTotal + Expr.Evaluate();
     x := x + 1;
   end;

   Elapsed := DateTimeToTimeStamp(Time - TimerStart);
   
   lblElapsed.Caption := 'Elapsed Time: ' + FloatToStr(Elapsed.Time/1000) + ' seconds';
   txtSumResult.Text  := FormatFloat('#', SumTotal);

   Expr.Release();
   VariableX.Release();
end;

procedure TForm1.btnEvalClick(Sender: TObject);
begin
   // ucEval() evaluates an expression and returns a numeric result.
   // ucEvalStr() can evaluate expressions of any data type (numeric, string, or other).
   // If the result is numeric ucEvalStr conveniently converts it to a string for you.
   txtResult.Text := uc.EvalStr(cmbExpression.Text);
end;

procedure TForm1.btnDefineClick(Sender: TObject);
begin
   // ucDefine can define a variable, function, operator, syntax construct,
   // constant, pattern, data type, and more.
   //
   // Examples:
   //
   // ucDefine('Var: MyVariable As String');
   // ucDefine('Func: Area(Length, Width) = Length * Width');
   // ucDefine('Op: 50 {x}%  :=  x / 100');   // 50 is an arbitrary precedence level.
   // ucDefine('Syntax: {a} + {a} ::= "Two {a}s are better than one {a}"');
   // ucDefine('Const: Pi = Atan(1) * 4');
   uc.Define(cmbDefinition.Text);
end;

procedure TForm1.btnPlotClick(Sender: TObject);
var
   x: Double;
   Eq: uCalc.ParsedExpr;
   VariableX: uCalc.Item;
begin
   Image1.Canvas.FillRect(ClientRect);
   VariableX := uc.DefineVariable('x', @x);
   Eq := uc.Parse(txtPlotEq.Text);

   W := Image1.Width;
   H := Image1.Height;
   x := -10;

   Image1.Canvas.MoveTo(0, H/2);

   while x <= 10 do
   begin try
      CartesianLineTo(x, Eq.Evaluate());
   except end;
      x := x + 0.125;
   end;

   Eq.Release;
   VariableX.Release;
end;

procedure TForm1.chkInvalidOpClick(Sender: TObject);
begin
   // If toggled, then an expression such as 0 / 0 raises an error.
   // If toggled again, then 0 / 0 returns NaN.
   uc.FPEnvToggleExceptionMask(FPEnvExceptionEnum.Invalid);
end;

procedure TForm1.chkDenormalOpClick(Sender: TObject);
begin
uc.FPEnvToggleExceptionMask(FPEnvExceptionEnum.Inexact);
end;

procedure TForm1.chkDivisionBy0Click(Sender: TObject);
begin
   // If toggled, then 1 / 0 raises an error.
   // If toggled again, then 1 / 0 returns Inf.
   uc.FPEnvToggleExceptionMask(FPEnvExceptionEnum.DivisionByZero);
end;

procedure TForm1.chkOverflowClick(Sender: TObject);
begin
   // If toggled, then 100! ^ 150 raises an error.
   // If toggled again, then 100! ^ 150 returns Inf.
   uc.FPEnvToggleExceptionMask(FPEnvExceptionEnum.Overflow);
end;

procedure TForm1.chkUnderflowClick(Sender: TObject);
begin
   // If toggled, then (1/100!)^150 raises an error.
   // If toggled again, then (1/100!)^150 returns 0.
   uc.FPEnvToggleExceptionMask(FPEnvExceptionEnum.Underflow);
end;

procedure TForm1.chkPrecisionLossClick(Sender: TObject);
begin
   // If toggled, an expression that cannot be represented in exact form
   // such as "1/3" will raise an error.
   // If toggled again, "1/3" returns the closest double precision approximation.
   uc.FPEnvToggleExceptionMask(FPEnvExceptionEnum.Inexact);
end;


end.
