#pragma once

#include <cstdint>
#include <string>
#if defined(_WIN64) || defined(_WIN32)
#include <wtypes.h>
#if _MSC_VER < 1900   
enum CFENV : int {FE_DOWNWARD=512, FE_TONEAREST=0, FE_TOWARDZERO=768, FE_UPWARD=256,
   FE_DIVBYZERO=8, FE_INEXACT=1, FE_INVALID=16, FE_OVERFLOW=4, FE_UNDERFLOW=2, FE_ALL_EXCEPT=31};
#else  
#include <cfenv>
#endif
#else  
#include <cfenv>
#include <dlfcn.h>
#endif

namespace uCalcNameSpace {
   using namespace std;
   
#if _WIN64
    #define uCalcLib "uCalc64.dll"
#elif _WIN32
    #define uCalcLib "uCalc32.dll"
#else  
    #define uCalcLib "./libuCalc.so"
    #define LoadLibraryA(lib) dlopen(lib, RTLD_LAZY)
    #define GetProcAddress(lib_handle, func) dlsym(lib_handle, func)
    #define HINSTANCE void *
    #define _stdcall /* Nothing */
#endif
   
#if defined(__CSTRINGT_H__)
    #define CONSTSTRING const CString &
    #define STRING CString
    #define RETURNSTRING(arg) return CString(arg)
    #define CNV(arg) CT2CA(arg)
   
#if defined(_MBCS)        
        #define EMPTYSTR ""
    #else        
        #define EMPTYSTR L""
    #endif
#else
    #define CONSTSTRING const string &
    #define STRING string
    #define RETURNSTRING(arg) return arg
    #define CNV(arg) arg.c_str()
    #define EMPTYSTR ""
#endif
   
#define BYREF &
#define UCCALLBACK uCalcCallback
#define UCALC_CALLBACK void _stdcall
#define CONSTCHAR const char *
   /* #define marker */
   
   typedef void (*uCalcPtr)(bool, char, int, long, float, double);
   typedef void (_stdcall *uCalcCallback)(uCalcPtr);
   
   /// <summary>S: Data types that come with uCalc</summary>
   /// <remarks>These are built-in data types that come with uCalc</remarks>
   enum BuiltInTypesEnum : int32_t {
      Type_Unspecified,
      Type_Auto,
      /// <summary> Boolean -- </summary>
      Type_Boolean,
      /// <summary> Int8 +++ Rename these to things like Type_Int8</summary>
      Type_Integer_8,
      /// <summary> Byte</summary>
      Type_Integer_8u,
      /// <summary> Int16</summary>
      Type_Integer_16,
      /// <summary> Int16u</summary>
      Type_Integer_16u,
      /// <summary> Int32</summary>
      Type_Integer_32,
      /// <summary> Int32u</summary>
      Type_Integer_32u,
      /// <summary> Int64</summary>
      Type_Integer_64,
      /// <summary> Int64u</summary>
      Type_Integer_64u,
      /// <summary> Int128</summary>
      Type_Integer_128,
      /// <summary> Int128u</summary>
      Type_Integer_128u,
      /// <summary> Single</summary>
      Type_Float_Single,
      /// <summary> Double</summary>
      Type_Float_Double,
      /// <summary> Extended</summary>
      Type_Float_Extended,
      Type_Size_t,
      Type_Complex,
      Type_Decimal,
      Type_BigNum,
      Type_Date,
      Type_String,
      /// <summary> LPCSTR</summary>
      Type_String_LPCSTR,
      /// <summary> FixedStr</summary>
      Type_String_Fixed,
      /// <summary> WideStr</summary>
      Type_String_Wide,
      /// <summary> StackList</summary>
      Type_List_Stack,
      /// <summary> TableList</summary>
      Type_List_Table,
      /// <summary> SortedList</summary>
      Type_List_Sorted,
      Type_Null,
      Type_Void,
      Type_Any,
      Type_Pointer,
      Type_Omni,
      Type_UserDefined,
      Type_DataType,
      Type_Namespace,
      Type_IntPtr,
      Type_ADDR,
      Type_uCalcPtr,
      Type_Item,
      Type_ParsedExpr,
      Type_uCalcCallback,
      Type_Object,
      Type_Intrinsic
   };
   
   /// <summary>S: Data type constructors</summary>
   enum DataTypeConstruction : int32_t {
      TypeConstr_ByteSize,
      TypeConstr_AllocArray,
      TypeConstr_AllocScalar,
      TypeConstr_AllocByPtr,
      TypeConstr_ReleaseArray,
      TypeConstr_ReleaseScalar,
      TypeConstr_ResetScalar,
      TypeConstr_ResetArray,
      TypeConstr_SetArrayValue,
      TypeConstr_SetScalarValue,
      TypeConstr_SwapScalarValues
   };
   
   /// <summary>S: Error hander responses</summary>
   enum ErrorHandlerResponseEnum : int32_t {
      /// <summary> Immediately stops parsing</summary>
      ErrorAbort,
      /// <summary> Continues parsing (after the error is resolved)</summary>
      ErrorResume,
      /// <summary> Raises the same error again for the next error handler to take care of</summary>
      ErrorReRaise
   };
   
   /// <summary>S: Error numbers that uCalc may return</summary>
   enum ErrorNumberEnum : int32_t {
      /// <summary> No error</summary>
      Err_None,
      /// <summary> Dynamically defined user error</summary>
      Err_Dynamically_Defined,
      /// <summary> Syntax error</summary>
      Err_Syntax_Error,
      /// <summary> Undefined identifier</summary>
      Err_Undefined_Identifier,
      /// <summary> File not found</summary>
      Err_FileNotFound,
      /// <summary> Unrecognized token</summary>
      Err_Unrecognized_Token,
      /// <summary> Unrecognized command</summary>
      Err_Unrecognized_Command,
      /// <summary> Data type mismatch</summary>
      Err_Datatype_Mismatch,
      /// <summary> Invalid number of arguments</summary>
      Err_Invalid_Argument_Count,
      /// <summary> Invalid definition</summary>
      Err_Invalid_Definition,
      /// <summary> Code block delimiter error</summary>
      Err_CodeBlock_Error,
      /// <summary> Undefined callback</summary>
      Err_Undefined_Callback,
      /// <summary> Error message already defined</summary>
      Err_ErrorMessageAlreadyDefined,
      /// <summary> This item cannot be modified</summary>
      Err_ItemCannotBeModified,
      /// <summary> Unrecognized class member</summary>
      Err_Unrecognized_Class_Member,
      /// <summary> Unrecognized namespace member</summary>
      Err_Unrecognized_Namespace_Member,
      /// <summary> Unbalanced quote</summary>
      Err_Unbalanced_Quote,
      /// <summary> Value cannot be assigned here</summary>
      Err_ValueCannotBeAssigned,
      /// <summary> Reparsing has passed the specified limit</summary>
      Err_ReParseOverflow,
      /// <summary> Array or list bound exceeded</summary>
      Err_Array_Bounds_Exceeded,
      /// <summary> Division by 0</summary>
      Err_Float_Division_By_Zero,
      /// <summary> Inexact result</summary>
      Err_Float_Inexact_Result,
      /// <summary> Invalid operation</summary>
      Err_Float_Invalid_Operation,
      /// <summary> Floating point overflow</summary>
      Err_Float_Overflow,
      /// <summary> Floating point underflow</summary>
      Err_Float_Underflow,
      Err_Customized1 = 100,
      Err_Customized2,
      Err_Customized3,
      Err_Customized4,
      Err_Customized5,
      Err_Customized6,
      Err_Customized7,
      Err_Customized8,
      Err_Customized9,
      Err_Customized10,
      Err_Customized11,
      Err_Customized12,
      Err_Customized13,
      Err_Customized14,
      Err_Customized15,
      Err_Customized16,
      Err_Customized17,
      Err_Customized18,
      Err_Customized19,
      Err_Customized20
   };
   
   /// <summary>S: Operator grouping associativity</summary>
   enum GroupingEnum : int32_t {
      GroupLeftToRight,
      GroupRightToLeft
   };
   
   /// <summary>S: Properties that a uCalc item may have</summary>
   /// <remarks>
   ///Items defined with uCalc typically have a combination of properties.  A variable defined
   ///with DefineVariable for instance will have the ItemIsEnum.Variable property set.  An operator
   ///will have the ItemIsEnum.Operator property set.  If it's an infix operator, in addition to
   ///having the ItemIsEnum.Operator property set, ItemIsEnum.Infix will also be set.
   ///property will also be set.
   ///
   ///The IsProperty function can determine if a given uCalc.Item has a certain property.
   ///
   ///You can use the GetItemOfProperty function to cycle through the list of all items that share
   ///certain properties, or to disambiguate between items that share the same name but have
   ///different roperties (for instance the + operator with the Prefix property vs another one
   ///with the Infix property).
   ///
   ///Some members are keywords in VB.NET.  To avoid naming conflict, the following members have been
   ///renamed in the VB.NET header: Function_, Operator_, Optional_, ByVal_, ByRef_, Class_, and NameSpace_.
   ///</remarks>
   enum ItemIsEnum : int32_t {
      AnyItem,
      RightToLeft,
      DataType,
      Array,
      Class,
      NameSpace,
      Variable,
      LocalVariable,
      Function,
      Operator,
      FunctionOrOperator,
      Prefix,
      Infix,
      Postfix,
      AddressOwner,
      ByVal,
      ByRef,
      ByCopy,
      ByExpr,
      ByHandle,
      Pointer,
      Optional,
      VariableNumberOfArgs,
      LiteralItem,
      Locked,
      Enabled,
      /// <summary> +++ May need to rename; Purpose of Exclude: cause match to fail if this search item is found   </summary>
      Exclude,
      Transform,
      Search,
      ItemDef,
      ErrorHandler,
      StopNow,
      ApiCall,
      CaseSensitive,
      QuotedText,
      /// <summary> +++ RewindOnModify, RewindOnUpdate, RewindOnAlter, RewindOnRevise, RewindOnRevision, RewindOnRewriteRewindOnEdit, other?</summary>
      RewindOnChange,
      ExpandArg,
      Active,
      IgnoreStatementSep,
      IgnoreCodeBlock,   
      SkipEvalExpand,
      ExprUnit,
      EagerArg,
      SkipSyntax,
      SkipOver,
      Modified,
      /// <summary> +++ Maybe rename WhiteSpaceSensitive   </summary>
      WhiteSpaceCounts,
      IgnoreQuote,
      UpToEndOrSeparator,
      UpToUserStop,
      UpToCompleted,   
      ExpandMode,
      StepsOn,
      EvalStrOutput,
      SkipOutput,
      DefaultSettings,
      NotFound,  
      MiscItem
   };
   
   /// <summary>S: Licensing options for uCalc products</summary>
   enum LicenseTypeEnum : int32_t {
      License_Demo,
      License_FMP_Free,
      License_FMP_Standard_32,
      License_FMP_Standard_64,
      License_FMP_32_and_64,
      License_LanguageBuilder,
      License_GeneralParser,
      License_Interpreter,
      License_ConsoleCalc,
      License_WindowsCalc,
      License_StringLib,
      License_Transform,
      License_TransformLib,
      License_Search,
      License_SearchLib,
      License_Library1,
      License_Library2,
      License_Beta,
      License_Debugger,
      License_uCalc_All,
      License_Developer,
      License_EndUser
   };
   
   /// <summary>S: Types of tokens</summary>
   enum TokenEnum : int32_t {
      None,
      /// <summary> A token defined without any specified property is generic (default)</summary>
      Generic,
      /// <summary> Represents the absence of any character (token not found)</summary>
      Empty,
      /// <summary> A token that behaves as alphanuric (works with word boundaries)</summary>
      AlphaNumeric,
      /// <summary> A token that serves as argument separator (typically a comma)</summary>
      ArgSeparator,
      /// <summary> A token that starts a code block; typically characters like {, [, or (</summary>
      CodeBlock,
      /// <summary> The closing token that matches a CodeBlock token, such as }, ], or )</summary>
      CodeBlockMatch,
      /// <summary> A token that separates an object from its member, typically a . (a dot)</summary>
      MemberAccess,
      /// <summary> A token that represents a literal value, typically a quoted string or number</summary>
      Literal,
      /// <summary> A token that represents a statement separator, typically a ; (semicolon)</summary>
      StatementSep,
      /// <summary> A token that represents whitespace, typically tab or space characters</summary>
      WhiteSpace,
      /// <summary> A token that is not delimited by word boundaries (+, -, *, etc), consecutive characters are reduced until they match a value that exists in the symbol table or until 1 character</summary>
      Reducible,
      /// <summary> Non-character that represents the end of a stream of text</summary>
      Eof,
      CurrentThread, 
      LocalVar,
      StaticVar,
      /// <summary> Item in parenthesis (or quotes) following this token will be defined while the expression is parsed</summary>
      DefineNow = StaticVar,
      TempDef,
      /// <summary> The expression in parathesis (or quotes) following the token is evaluated and inserted in place of the token in the string</summary>
      EvalInsert,
      EvalDuringParse,
      /// <summary> Works like EvalInsert, except the expression is expanded instead of evaluated</summary>
      ExpandInsert,
      /// <summary> The contents of the quoted file name following this token is inserted into the string</summary>
      FileInclude,
      CurrentSpace,
      DefSpace,
      EvalSpace
   };
   
   /// <summary>S: Boolean True or False</summary>
   enum TrueFalseEnum : int32_t {
      /// <summary> For compilers in which False is a keyword use False_ instead</summary>
      False,
      /// <summary> For compilers in which True is a keyword use True_ instead</summary>
      True
   };
   
   /// <summary>S: Floating point environment exceptions</summary>
   enum FPEnvExceptionEnum : int32_t {
      /// <summary> Floating point division by zero, or operation resulting in infinity, occured (pole error)</summary>
      DivisionByZero = FE_DIVBYZERO,
      /// <summary> The result is not an exact floating point number</summary>
      Inexact = FE_INEXACT,
      /// <summary> Argument passed to a function or operator was invalid (domain error)</summary>
      Invalid = FE_INVALID,
      /// <summary> The result is too large to be represented as a floating point value of the return type (overflow range error)</summary>
      Overflow = FE_OVERFLOW,
      /// <summary> The result is too small to be represented as a floating point value of the return type, causing loss of precision (underflow range error)</summary>
      Underflow = FE_UNDERFLOW,
      /// <summary> Represents selection of all the floating point exceptions: DivisionByZero, Inexact, Invalid, Overflow, and Underflow (Bitwise ORed)</summary>
      AllExceptions = FE_ALL_EXCEPT
   };
   
   /// <summary>S: Floating point rouding options</summary>
   enum FPEnvRoundingEnum : int32_t {
      /// <summary> Rounds downward (towards negative infinity)</summary>
      Round_Down = FE_DOWNWARD,
      /// <summary> Rounds towards nearest value</summary>
      Round_ToNearest = FE_TONEAREST,
      /// <summary> Rounds towards zero</summary>
      Round_TowardZero = FE_TOWARDZERO,
      /// <summary> Rounds upward (towards positive infinity)</summary>
      Round_Up = FE_UPWARD
   };
   
   /// <summary>S: RegEx grammar option for defining tokens; based on C++ regex flags</summary>
   enum RegExGrammarEnum : int32_t {
      /// <summary> Default grammar; ECMAScript is default unless reconfigured with +++</summary>
      RegEx_Default,
      /// <summary> JavaScript-like regex grammar</summary>
      RegEx_ECMAScript,
      /// <summary> Basic POSIX grammar (BRE)</summary>
      RegEx_basic,
      /// <summary> Extended POSIX grammar (ERE)</summary>
      RegEx_extended,
      /// <summary> awk POSIX grammar</summary>
      RegEx_awk,
      /// <summary> grep POSIX grammar</summary>
      RegEx_grep,
      /// <summary> egrep POXIX grammar</summary>
      RegEx_egrep
   };
   
   /// <summary>S: Transform reset options</summary>
   /// <remarks>Use these options with Transform Reset.  The Default is Reset_All</remarks>
   enum TransformResetEnum : int32_t {
      /// <summary> Resets all (Input, Tokens, Rules)</summary>
      Reset_All,
      /// <summary> Resets the input to an empty string</summary>
      Reset_Input,
      /// <summary> Resets token definitions</summary>
      Reset_Tokens,
      /// <summary> Resets the FromTo rules</summary>
      Reset_Rules,
      /// <summary> Resets transform results</summary>
      Reset_Results,
      /// <summary> Removes passes</summary>
      Reset_Passes
   };
   
   /// <summary>S: Options for Transform Pass</summary>
   enum PassEnum : int32_t {
      /// <summary> Creates a pass with new token definitions that are separate from those of the parent +++ and separate one for transforms</summary>
      Pass_Independent,
      /// <summary> Creates a pass that inherits a separate copy of the token list from the parent</summary>
      Pass_InheritCopy,
      /// <summary> Creates a pass with that shares the same tokens as the parent</summary>
      Pass_InheritShare
   };
   
   /// <summary>S: Options for transform steps</summary>
   enum StepsEnum : int32_t {
      Steps_None,
      Steps_All,
      Steps_Passes
   };
   
   /// <summary>S: Transform Properties</summary>
   enum TransformPropertiesEnum : int32_t {
      NoTransformProperties,
      /// <summary> Save snapshots of entire text for each change</summary>
      Snapshot_EachChange = 1,
      /// <summary> Save snapshots of entire text for each pass</summary>
      Snapshot_EachPass = 2,
      /// <summary> Save snapshots of each local change</summary>
      Snapshot_LocalChange = 4,
      /// <summary> Save snapshots of local text before each change</summary>
      Archive_LocalOriginals = 5,
      /// <summary> Count number of modifications made by each rule</summary>
      CountChanges = 8
   };
   
   /* TypeDef */
   typedef double (* __Evaluate)(void *ExprHandle); extern __Evaluate Evaluate_;
   typedef double (* __EvaluateNTE)(void *ExprHandle); extern __EvaluateNTE EvaluateNTE_;
   typedef int32_t (* __EvaluateInt32)(void *ExprHandle); extern __EvaluateInt32 EvaluateInt32_;
   typedef const char * (* __EvaluateStr)(void *ExprHandle); extern __EvaluateStr EvaluateStr_;
   typedef void * (* __EvaluateVoid)(void *ExprHandle); extern __EvaluateVoid EvaluateVoid_;
   typedef void (* __Execute)(void *ExprHandle); extern __Execute Execute_;
   typedef void (* __ReleaseExpr)(void *ExprHandle); extern __ReleaseExpr ReleaseExpr_;
   typedef void * (* __Handle_Overload_ParsedExpr)(void *ExprHandle); extern __Handle_Overload_ParsedExpr Handle_Overload_ParsedExpr_;
   typedef int32_t (* __ElementCount)(void *ItemHandle); extern __ElementCount ElementCount_;
   typedef uCalcPtr (* __GetDataType)(void *ItemHandle); extern __GetDataType GetDataType_;
   typedef uCalcCallback (* __FunctionAddress)(void *ItemHandle); extern __FunctionAddress FunctionAddress_;
   typedef int32_t (* __IsProperty)(void *ItemHandle, ItemIsEnum ItemProperty); extern __IsProperty IsProperty_;
   typedef const char * (* __Name_Overload_Item)(void *ItemHandle); extern __Name_Overload_Item Name_Overload_Item_;
   typedef uCalcPtr (* __NextOverload)(void *ItemHandle); extern __NextOverload NextOverload_;
   typedef int32_t (* __Precedence_Overload_Item)(void *ItemHandle); extern __Precedence_Overload_Item Precedence_Overload_Item_;
   typedef void (* __ReleaseItem)(void *ItemHandle); extern __ReleaseItem ReleaseItem_;
   typedef void (* __Rename)(void *ItemHandle, CONSTCHAR NewName ); extern __Rename Rename_;
   typedef void (* __SetVariableValue)(void *ItemHandle, double Value); extern __SetVariableValue SetVariableValue_;
   typedef void (* __SetVariableValueAny)(void *ItemHandle, CONSTCHAR Value ); extern __SetVariableValueAny SetVariableValueAny_;
   typedef void (* __SetVariableValueByte)(void *ItemHandle, uint8_t Value); extern __SetVariableValueByte SetVariableValueByte_;
   typedef void (* __SetVariableValueInt16)(void *ItemHandle, int16_t Value); extern __SetVariableValueInt16 SetVariableValueInt16_;
   typedef void (* __SetVariableValueInt32)(void *ItemHandle, int32_t Value); extern __SetVariableValueInt32 SetVariableValueInt32_;
   typedef void (* __SetVariableValueInt64)(void *ItemHandle, int64_t Value); extern __SetVariableValueInt64 SetVariableValueInt64_;
   typedef void (* __SetVariableValueSng)(void *ItemHandle, float Value); extern __SetVariableValueSng SetVariableValueSng_;
   typedef void (* __SetVariableValueDbl)(void *ItemHandle, double Value); extern __SetVariableValueDbl SetVariableValueDbl_;
   typedef void (* __SetVariableValueStr)(void *ItemHandle, CONSTCHAR Value ); extern __SetVariableValueStr SetVariableValueStr_;
   typedef void (* __SetVariableValuePtr)(void *ItemHandle, void * Value); extern __SetVariableValuePtr SetVariableValuePtr_;
   typedef void (* __SetVariableValueByPtr)(void *ItemHandle, void * Value); extern __SetVariableValueByPtr SetVariableValueByPtr_;
   typedef const char * (* __Text)(void *ItemHandle); extern __Text Text_;
   typedef uCalcPtr (* __GetuCalc_Overload_Item)(void *ItemHandle); extern __GetuCalc_Overload_Item GetuCalc_Overload_Item_;
   typedef double (* __Value)(void *ItemHandle); extern __Value Value_;
   typedef uint8_t (* __ValueByte)(void *ItemHandle); extern __ValueByte ValueByte_;
   typedef int16_t (* __ValueInt16)(void *ItemHandle); extern __ValueInt16 ValueInt16_;
   typedef int32_t (* __ValueInt32)(void *ItemHandle); extern __ValueInt32 ValueInt32_;
   typedef int64_t (* __ValueInt64)(void *ItemHandle); extern __ValueInt64 ValueInt64_;
   typedef float (* __ValueSng)(void *ItemHandle); extern __ValueSng ValueSng_;
   typedef double (* __ValueDbl)(void *ItemHandle); extern __ValueDbl ValueDbl_;
   typedef const char * (* __ValueStr)(void *ItemHandle); extern __ValueStr ValueStr_;
   typedef void * (* __ValuePtr)(void *ItemHandle); extern __ValuePtr ValuePtr_;
   typedef void * (* __ValueAddr)(void *ItemHandle); extern __ValueAddr ValueAddr_;
   typedef void * (* __Handle_Overload_Item)(void *ItemHandle); extern __Handle_Overload_Item Handle_Overload_Item_;
   typedef double (* __Arg)(void *ExprPartHandle, int32_t Index); extern __Arg Arg_;
   typedef int32_t (* __ArgCount)(void *ExprPartHandle); extern __ArgCount ArgCount_;
   typedef uCalcPtr (* __ArgObj)(void *ExprPartHandle, int32_t Index); extern __ArgObj ArgObj_;
   typedef void * (* __ArgAddr)(void *ExprPartHandle, int32_t Index); extern __ArgAddr ArgAddr_;
   typedef uCalcPtr (* __ArgExpr)(void *ExprPartHandle, int32_t Index); extern __ArgExpr ArgExpr_;
   typedef int32_t (* __ArgInt32)(void *ExprPartHandle, int32_t Index); extern __ArgInt32 ArgInt32_;
   typedef int64_t (* __ArgInt64)(void *ExprPartHandle, int32_t Index); extern __ArgInt64 ArgInt64_;
   typedef void * (* __ArgPtr)(void *ExprPartHandle, int32_t Index); extern __ArgPtr ArgPtr_;
   typedef const char * (* __ArgStr)(void *ExprPartHandle, int32_t Index); extern __ArgStr ArgStr_;
   typedef void (* __Return)(void *ExprPartHandle, double Value); extern __Return Return_;
   typedef void (* __ReturnInt32)(void *ExprPartHandle, int32_t Value); extern __ReturnInt32 ReturnInt32_;
   typedef void (* __ReturnInt64)(void *ExprPartHandle, int64_t Value); extern __ReturnInt64 ReturnInt64_;
   typedef void (* __ReturnPtr)(void *ExprPartHandle, void * Value); extern __ReturnPtr ReturnPtr_;
   typedef void (* __ReturnStr)(void *ExprPartHandle, CONSTCHAR Value ); extern __ReturnStr ReturnStr_;
   typedef int32_t (* __ErrorRaise)(void *ExprPartHandle, ErrorNumberEnum ErrNumber); extern __ErrorRaise ErrorRaise_;
   typedef int32_t (* __ErrorRaiseMessage)(void *ExprPartHandle, CONSTCHAR ErrMessage ); extern __ErrorRaiseMessage ErrorRaiseMessage_;
   typedef uCalcPtr (* __GetuCalc_Overload_Callback)(void *ExprPartHandle); extern __GetuCalc_Overload_Callback GetuCalc_Overload_Callback_;
   typedef void * (* __Handle_Overload_Callback)(void *ExprPartHandle); extern __Handle_Overload_Callback Handle_Overload_Callback_;
   typedef uCalcPtr (* __OpItem)(void *ExprPartHandle); extern __OpItem OpItem_;
   typedef void * (* __AllocArray)(void *DataTypeHandle, int32_t ArraySize); extern __AllocArray AllocArray_;
   typedef void * (* __AllocByPtr)(void *DataTypeHandle, void * ValuePtr); extern __AllocByPtr AllocByPtr_;
   typedef void * (* __AllocScalar)(void *DataTypeHandle,  CONSTCHAR Value   ); extern __AllocScalar AllocScalar_;
   typedef int32_t (* __ByteSize)(void *DataTypeHandle); extern __ByteSize ByteSize_;
   typedef BuiltInTypesEnum (* __Index)(void *DataTypeHandle); extern __Index Index_;
   typedef uCalcPtr (* __GetItem)(void *DataTypeHandle); extern __GetItem GetItem_;
   typedef const char * (* __Name_Overload_DataType)(void *DataTypeHandle); extern __Name_Overload_DataType Name_Overload_DataType_;
   typedef void (* __ReleaseArray)(void *DataTypeHandle, void * ValuePtr); extern __ReleaseArray ReleaseArray_;
   typedef void (* __ReleaseScalar)(void *DataTypeHandle, void * ValuePtr); extern __ReleaseScalar ReleaseScalar_;
   typedef void (* __ResetArray)(void *DataTypeHandle, void * ValuePtr); extern __ResetArray ResetArray_;
   typedef void (* __ResetScalar)(void *DataTypeHandle, void * ValuePtr); extern __ResetScalar ResetScalar_;
   typedef void (* __SetArray)(void *DataTypeHandle, void * Destination ,  void * Source ,  int32_t Index); extern __SetArray SetArray_;
   typedef void (* __SetScalar)(void *DataTypeHandle, void * Destination ,  void * Source); extern __SetScalar SetScalar_;
   typedef void (* __SwapScalarValues)(void *DataTypeHandle, void * ValuePtrA ,  void * ValuePtrB); extern __SwapScalarValues SwapScalarValues_;
   typedef const char * (* __ToString_Overload_DataType__ByPtr)(void *DataTypeHandle, void * ValuePtr); extern __ToString_Overload_DataType__ByPtr ToString_Overload_DataType__ByPtr_;
   typedef const char * (* __ToString_Overload_DataType__ByStr)(void *DataTypeHandle, CONSTCHAR Value ); extern __ToString_Overload_DataType__ByStr ToString_Overload_DataType__ByStr_;
   typedef void * (* __Handle_Overload_DataType)(void *DataTypeHandle); extern __Handle_Overload_DataType Handle_Overload_DataType_;
   typedef uCalcPtr (* __Active)(void *RuleHandle, bool TrueFalse); extern __Active Active_;
   typedef uCalcPtr (* __FindPart)(void *RuleHandle, CONSTCHAR Pattern ); extern __FindPart FindPart_;
   typedef uCalcPtr (* __FromPart)(void *RuleHandle, CONSTCHAR Pattern ); extern __FromPart FromPart_;
   typedef uCalcPtr (* __ToPart)(void *RuleHandle, CONSTCHAR Pattern ); extern __ToPart ToPart_;
   typedef uCalcPtr (* __CaseSensitive)(void *RuleHandle, bool TrueFalse); extern __CaseSensitive CaseSensitive_;
   typedef uCalcPtr (* __WhitespaceSensitive)(void *RuleHandle, bool TrueFalse); extern __WhitespaceSensitive WhitespaceSensitive_;
   typedef uCalcPtr (* __CodeBlockSensitive)(void *RuleHandle, bool TrueFalse); extern __CodeBlockSensitive CodeBlockSensitive_;
   typedef uCalcPtr (* __Comment_Overload_uCalcTransform_RuleClass)(void *RuleHandle, CONSTCHAR CommentStr ); extern __Comment_Overload_uCalcTransform_RuleClass Comment_Overload_uCalcTransform_RuleClass_;
   typedef uCalcPtr (* __LocalTransform)(void *RuleHandle); extern __LocalTransform LocalTransform_;
   typedef uCalcPtr (* __Maximum)(void *RuleHandle, int32_t Value); extern __Maximum Maximum_;
   typedef uCalcPtr (* __MaximumHard)(void *RuleHandle, int32_t Value); extern __MaximumHard MaximumHard_;
   typedef uCalcPtr (* __Minimum)(void *RuleHandle, int32_t Value); extern __Minimum Minimum_;
   typedef uCalcPtr (* __MinimumHard)(void *RuleHandle, int32_t Value); extern __MinimumHard MinimumHard_;
   typedef uCalcPtr (* __Precedence)(void *RuleHandle, int32_t Value); extern __Precedence Precedence_;
   typedef uCalcPtr (* __QuoteSensitive)(void *RuleHandle, bool TrueFalse); extern __QuoteSensitive QuoteSensitive_;
   typedef const char * (* __ReplacementText)(void *RuleHandle); extern __ReplacementText ReplacementText_;
   typedef uCalcPtr (* __StartAfter)(void *RuleHandle, int32_t Value); extern __StartAfter StartAfter_;
   typedef uCalcPtr (* __StopAfter)(void *RuleHandle, int32_t Value); extern __StopAfter StopAfter_;
   typedef uCalcPtr (* __SkipOver)(void *RuleHandle, bool TrueFalse); extern __SkipOver SkipOver_;
   typedef uCalcPtr (* __RewindOnChange)(void *RuleHandle, bool TrueFalse); extern __RewindOnChange RewindOnChange_;
   typedef uCalcPtr (* __ParseAgainMax)(void *RuleHandle, bool TrueFalse ,  bool RaiseError); extern __ParseAgainMax ParseAgainMax_;
   typedef uCalcPtr (* __RightToLeft)(void *RuleHandle, bool TrueFalse); extern __RightToLeft RightToLeft_;
   typedef void (* __ReleaseRule)(void *RuleHandle); extern __ReleaseRule ReleaseRule_;
   typedef uCalcPtr (* __Comment_Overload_uCalcTransform_Tokens)(void *TokensHandle, CONSTCHAR CommentStr ); extern __Comment_Overload_uCalcTransform_Tokens Comment_Overload_uCalcTransform_Tokens_;
   typedef uCalcPtr (* __Import_Overload_uCalcTransform_Tokens__Obj)(void *TokensHandle, void * FromTokenList); extern __Import_Overload_uCalcTransform_Tokens__Obj Import_Overload_uCalcTransform_Tokens__Obj_;
   typedef uCalcPtr (* __Import_Overload_uCalcTransform_Tokens__File)(void *TokensHandle, CONSTCHAR TokenFile ); extern __Import_Overload_uCalcTransform_Tokens__File Import_Overload_uCalcTransform_Tokens__File_;
   typedef uCalcPtr (* __Define_Overload_uCalcTransform_Tokens)(void *TokensHandle, CONSTCHAR TokenDef  ,  TokenEnum TokenType  ,  CONSTCHAR EndMatch   ,  int32_t SubMatch  ,  RegExGrammarEnum RegExGrammar  ); extern __Define_Overload_uCalcTransform_Tokens Define_Overload_uCalcTransform_Tokens_;
   typedef uCalcPtr (* __Comment_Overload_uCalcTransform)(void *TransformHandle, CONSTCHAR CommentStr ); extern __Comment_Overload_uCalcTransform Comment_Overload_uCalcTransform_;
   typedef uCalcPtr (* __DefineRule)(void *TransformHandle, CONSTCHAR Comment  ,  CONSTCHAR FindStr  ,  CONSTCHAR ToStr  ,  int32_t Min ,  int32_t Max ,  int32_t MinHard ,  int32_t MaxHard ,  int32_t Precedence ,  int32_t StartAfter ,  int32_t StopAfter ,  bool SkipOver ,  bool RewindOnChange ,  bool CaseSensitive ,  bool CodeblockSensitive ,  bool QuoteSensitive ,  bool RightToLeft); extern __DefineRule DefineRule_;
   typedef uCalcPtr (* __DefineToken)(void *TransformHandle, CONSTCHAR TokenDef  ,  TokenEnum TokenType  ,  CONSTCHAR EndMatch   ,  int32_t SubMatch  ,  RegExGrammarEnum RegExGrammar  ); extern __DefineToken DefineToken_;
   typedef uCalcPtr (* __GetTokens)(void *TransformHandle); extern __GetTokens GetTokens_;
   typedef uCalcPtr (* __FromTo)(void *TransformHandle, CONSTCHAR FromStr  ,  CONSTCHAR ToStr   ); extern __FromTo FromTo_;
   typedef uCalcPtr (* __Import_Overload_uCalcTransform)(void *TransformHandle, CONSTCHAR FileName ); extern __Import_Overload_uCalcTransform Import_Overload_uCalcTransform_;
   typedef uCalcPtr (* __ListPrefix)(void *TransformHandle, CONSTCHAR Prefix ); extern __ListPrefix ListPrefix_;
   typedef uCalcPtr (* __ListPostfix)(void *TransformHandle, CONSTCHAR Postfix ); extern __ListPostfix ListPostfix_;
   typedef uCalcPtr (* __ListSeparator)(void *TransformHandle, CONSTCHAR Separator ); extern __ListSeparator ListSeparator_;
   typedef uCalcPtr (* __ListDelimiters)(void *TransformHandle, CONSTCHAR Prefix  ,  CONSTCHAR Separator  ,  CONSTCHAR Postfix ); extern __ListDelimiters ListDelimiters_;
   typedef uCalcPtr (* __MathParser)(void *TransformHandle,  void * FMP  ); extern __MathParser MathParser_;
   typedef uCalcPtr (* __NewPass)(void *TransformHandle,  bool Share  ); extern __NewPass NewPass_;
   typedef uCalcPtr (* __Pass)(void *TransformHandle, int32_t Index ,  bool Share  ); extern __Pass Pass_;
   typedef uCalcPtr (* __Reset)(void *TransformHandle,  TransformResetEnum ResetOption  ); extern __Reset Reset_;
   typedef const char * (* __ResultStr)(void *TransformHandle); extern __ResultStr ResultStr_;
   typedef uCalcPtr (* __Transform)(void *TransformHandle); extern __Transform Transform_;
   typedef const char * (* __Expand)(void *TransformHandle, CONSTCHAR Expression ); extern __Expand Expand_;
   typedef const char * (* __ExpandWithSteps)(void *TransformHandle, CONSTCHAR Expression  ,  CONSTCHAR Separator   ,  CONSTCHAR Prefix   ,  CONSTCHAR Postfix   ); extern __ExpandWithSteps ExpandWithSteps_;
   typedef uCalcPtr (* __SetInput)(void *TransformHandle, CONSTCHAR InputStr ); extern __SetInput SetInput_;
   typedef uCalcPtr (* __SetInputByRef)(void *TransformHandle, void * InputStr); extern __SetInputByRef SetInputByRef_;
   typedef uCalcPtr (* __SetParser)(void *TransformHandle, void * FMP); extern __SetParser SetParser_;
   typedef uCalcPtr (* __SetProperties)(void *TransformHandle,  TransformPropertiesEnum TransformProperties  ); extern __SetProperties SetProperties_;
   typedef int32_t (* __WasModified)(void *TransformHandle); extern __WasModified WasModified_;
   typedef void (* __ReleaseTransform)(void *TransformHandle); extern __ReleaseTransform ReleaseTransform_;
   typedef uCalcPtr (* __GetRule)(void *TransformHandle,  int32_t Index  ); extern __GetRule GetRule_;
   typedef uCalcPtr (* __RulesDefault)(void *TransformHandle); extern __RulesDefault RulesDefault_;
   typedef uCalcPtr (* __RulesOverall)(void *TransformHandle); extern __RulesOverall RulesOverall_;
   typedef uCalcPtr (* __AddErrorHandler)(void *uCalcHandle, UCCALLBACK ErrorHandlerAddress ,  int32_t Position  ); extern __AddErrorHandler AddErrorHandler_;
   typedef uCalcPtr (* __CreateuCalc)(void *uCalcHandle); extern __CreateuCalc CreateuCalc_;
   typedef uCalcPtr (* __DataTypeOfIndex)(void *uCalcHandle, BuiltInTypesEnum Index); extern __DataTypeOfIndex DataTypeOfIndex_;
   typedef uCalcPtr (* __Define_Overload_uCalc)(void *uCalcHandle, CONSTCHAR Expression  ,  void * Address  ,  void * ExistingItem  ); extern __Define_Overload_uCalc Define_Overload_uCalc_;
   typedef uCalcPtr (* __DefineConstant)(void *uCalcHandle, CONSTCHAR Definition  ,  void * VariableAddress  ); extern __DefineConstant DefineConstant_;
   typedef uCalcPtr (* __DefineDataType)(void *uCalcHandle, int32_t TypeSize ,  void * AllocArrayValue ,  void * AllocScalar ,  void * AllocScalarByPtr ,  void * DeleteArray ,  void * DeleteScalar ,  void * ResetScalar ,  void * ResetArray ,  void * SetValueArray ,  void * SetValueScalar ,  void * SwapValues ,  void * ToString); extern __DefineDataType DefineDataType_;
   typedef uCalcPtr (* __DefineFunction)(void *uCalcHandle, CONSTCHAR Definition  ,  UCCALLBACK FunctionAddress  ); extern __DefineFunction DefineFunction_;
   typedef uCalcPtr (* __DefineOperator)(void *uCalcHandle, CONSTCHAR Definition  ,  int32_t iPrecedence ,  GroupingEnum Grouping  ,  UCCALLBACK FunctionAddress  ); extern __DefineOperator DefineOperator_;
   typedef uCalcPtr (* __DefineVariable)(void *uCalcHandle, CONSTCHAR Definition  ,  void * VariableAddress  ); extern __DefineVariable DefineVariable_;
   typedef const char * (* __ErrorExpression)(void *uCalcHandle); extern __ErrorExpression ErrorExpression_;
   typedef int32_t (* __ErrorLocation)(void *uCalcHandle); extern __ErrorLocation ErrorLocation_;
   typedef const char * (* __ErrorMessage)(void *uCalcHandle,  ErrorNumberEnum Index  ); extern __ErrorMessage ErrorMessage_;
   typedef ErrorNumberEnum (* __ErrorNumber)(void *uCalcHandle); extern __ErrorNumber ErrorNumber_;
   typedef void (* __ErrorResponse)(void *uCalcHandle, ErrorHandlerResponseEnum Response); extern __ErrorResponse ErrorResponse_;
   typedef const char * (* __ErrorSymbol)(void *uCalcHandle); extern __ErrorSymbol ErrorSymbol_;
   typedef void (* __ErrorSetExpression)(void *uCalcHandle, CONSTCHAR Expression ); extern __ErrorSetExpression ErrorSetExpression_;
   typedef double (* __Eval)(void *uCalcHandle, CONSTCHAR Expression ); extern __Eval Eval_;
   typedef const char * (* __EvalStr)(void *uCalcHandle, CONSTCHAR Expression ); extern __EvalStr EvalStr_;
   typedef int32_t (* __FPEnvClearException)(void *uCalcHandle,  int32_t Exceptions  ); extern __FPEnvClearException FPEnvClearException_;
   typedef int32_t (* __FPEnvGetExceptionMask)(void *uCalcHandle); extern __FPEnvGetExceptionMask FPEnvGetExceptionMask_;
   typedef int32_t (* __FPEnvRaiseException)(void *uCalcHandle,  FPEnvExceptionEnum Exception  ); extern __FPEnvRaiseException FPEnvRaiseException_;
   typedef FPEnvRoundingEnum (* __FPEnvGetRounding)(void *uCalcHandle); extern __FPEnvGetRounding FPEnvGetRounding_;
   typedef void (* __FPEnvSetExceptionMask)(void *uCalcHandle, int32_t Exceptions); extern __FPEnvSetExceptionMask FPEnvSetExceptionMask_;
   typedef void (* __FPEnvSetExceptionUnMask)(void *uCalcHandle, int32_t Exceptions); extern __FPEnvSetExceptionUnMask FPEnvSetExceptionUnMask_;
   typedef int32_t (* __FPEnvSetRounding)(void *uCalcHandle, FPEnvRoundingEnum Direction); extern __FPEnvSetRounding FPEnvSetRounding_;
   typedef void (* __FPEnvToggleExceptionMask)(void *uCalcHandle, FPEnvExceptionEnum Exception); extern __FPEnvToggleExceptionMask FPEnvToggleExceptionMask_;
   typedef uCalcPtr (* __HandleOf)(void *uCalcHandle, CONSTCHAR ItemName  ,  int64_t Properties  ,  bool SkipTransform  ); extern __HandleOf HandleOf_;
   typedef uCalcPtr (* __GetItemOf)(void *uCalcHandle, CONSTCHAR ItemName  ,  int64_t Properties  ,  bool SkipExpansion  ); extern __GetItemOf GetItemOf_;
   typedef uCalcPtr (* __GetItemOfProperty)(void *uCalcHandle, ItemIsEnum Property_ ,  int32_t nth ,  CONSTCHAR DataTypeName   ); extern __GetItemOfProperty GetItemOfProperty_;
   typedef int32_t (* __License)(void *uCalcHandle, LicenseTypeEnum LicenseType,   int32_t L1 ,   int32_t L2 ,   int32_t L3 ,   int32_t L4 ,   int32_t L5 ); extern __License License_;
   typedef uCalcPtr (* __NamespaceChild)(void *uCalcHandle,  int32_t nth ); extern __NamespaceChild NamespaceChild_;
   typedef int32_t (* __NamespaceChildCount)(void *uCalcHandle); extern __NamespaceChildCount NamespaceChildCount_;
   typedef uCalcPtr (* __NamespaceParent)(void *uCalcHandle,  int32_t Level ); extern __NamespaceParent NamespaceParent_;
   typedef uCalcPtr (* __NamespaceSymbol)(void *uCalcHandle, int32_t nth); extern __NamespaceSymbol NamespaceSymbol_;
   typedef int32_t (* __NamespaceSymbolCount)(void *uCalcHandle); extern __NamespaceSymbolCount NamespaceSymbolCount_;
   typedef uCalcPtr (* __NewTransform)(void *uCalcHandle); extern __NewTransform NewTransform_;
   typedef uCalcPtr (* __ParserTransform)(void *uCalcHandle); extern __ParserTransform ParserTransform_;
   typedef uCalcPtr (* __Parse)(void *uCalcHandle, CONSTCHAR Expression  ,  CONSTCHAR DataType   ); extern __Parse Parse_;
   typedef int32_t (* __Precedence_Overload_uCalc)(void *uCalcHandle, CONSTCHAR Operator1  ,  CONSTCHAR Operator2   ); extern __Precedence_Overload_uCalc Precedence_Overload_uCalc_;
   typedef int64_t (* __Properties)(void *uCalcHandle, ItemIsEnum p1,   ItemIsEnum p2 ,   ItemIsEnum p3 ,   ItemIsEnum p4 ); extern __Properties Properties_;
   typedef void (* __ReleaseuCalc)(void *uCalcHandle); extern __ReleaseuCalc ReleaseuCalc_;
   typedef void (* __ResetOutput)(void *uCalcHandle); extern __ResetOutput ResetOutput_;
   typedef uCalcPtr (* __Default)(void *uCalcHandle); extern __Default Default_;
   typedef uCalcPtr (* __SetOutput_Overload_uCalc__Callback)(void *uCalcHandle, UCCALLBACK FunctionAddress ,  CONSTCHAR Properties ); extern __SetOutput_Overload_uCalc__Callback SetOutput_Overload_uCalc__Callback_;
   typedef uCalcPtr (* __SetOutput_Overload_uCalc__NoCallback)(void *uCalcHandle, CONSTCHAR Definition ); extern __SetOutput_Overload_uCalc__NoCallback SetOutput_Overload_uCalc__NoCallback_;
   typedef uCalcPtr (* __CreateTransform)(void *uCalcHandle); extern __CreateTransform CreateTransform_;
   typedef const char * (* __ValueAt)(void *uCalcHandle, void * ValuePtr ,  CONSTCHAR DataType ); extern __ValueAt ValueAt_;
   typedef void * (* __Handle_Overload_uCalc)(void *uCalcHandle); extern __Handle_Overload_uCalc Handle_Overload_uCalc_;
   
   /// <summary> uCalc main class</summary>
   struct uCalc {
      struct ParsedExpr;
      struct Item;
      struct Callback;
      struct DataType;
      struct uCalcTransform;
      struct RuleClass;
      struct Tokens;
      
      /// <summary> Handle for uCalc instance</summary>
      uCalcPtr uCalcHandle;
      /// <summary> Class for evaluating an already parsed expression or executing parsed code</summary>
      struct ParsedExpr {
         
         uCalcPtr ExprHandle;
         
         /// <summary> Rapidly evaluates a floating point expression that was parsed with Parse or passed By Expression</summary>
         /// <returns> The result of the evaluated floating point expression</returns>
         /// <remarks><para>Evaluate can be used instead of Eval for faster results in time-critical loops where the same expression
         /// is repeatedly re-evaluated, with the only change in each iteration typically being the value(s) of one or
         /// more variables.  Evaluate works in pair with Parse, the relatively slower part, which is called first before
         /// going into the time-critical loop.  The ParsedExpr object returned by Parse is what calls the Evaluate function.
         /// </para><para>
         /// Evaluate can also be called by a ParsedExpr object that was passed to a callback By Expression.
         /// </para><para>
         /// The expression must be one who&#39;s evaluation returns a floating point numeric value.  If the expression evaluates
         /// to a non-numeric value (such as a string) or non-scalar numeric value (such as a complex number), or if a numeric
         /// type other than the default was explicitely specified with Parse, the value returned will be invalid.  However,
         /// an error will not be raised.
         /// </para></remarks> 
         double Evaluate() { return  Evaluate_(ExprHandle);          }
         
         /// <summary> Same as Evaluate, but with no testing or handling for floating point exceptions</summary>
         /// <returns> The result of the evaluated floating point expression</returns>
         /// <remarks><para>This is similar to Evaluate, with the difference being that EvaluateNTE does not perform any exception handling
         /// test.  As a consequence, the uCalc error handler callback does not handle these exceptions.
         /// </para><para>
         /// This option can speed up evaluations.  However, it&#39;s important to have a plan for handling exceptions.  You can either
         /// mask all exceptions with FPEnvSetExceptionMask or FPEnvToggleExceptionMask, or catch exceptions with your own source
         /// code (such as with try/catch).  If such provisions are not met, floating point exceptions (such as division by 0), may
         /// cause your program to behave unexpectadly or to terminate abruptly.
         /// </para></remarks> 
         double EvaluateNTE() { return  EvaluateNTE_(ExprHandle);          }
         
         /// <summary> Rapidly evaluates expression that was parsed as integer with Parse or passed By Expression</summary>
         /// <returns> The result of the evaluated integer expression</returns>
         /// <remarks><para>
         /// This is similar to Evaluate, but applies only to integer expressions.
         /// </para></remarks> 
         int32_t EvaluateInt32() { return  EvaluateInt32_(ExprHandle);          }
         
         /// <summary> Rapidly evaluates an expression of any data type that was parsed with Parse or passed By Expression returning a string</summary>
         /// <returns> The result of an evaluated expression of any data type, returned as a string</returns>
         /// <remarks><para>EvaluateStr is similar to Evaluate.  However, EvaluateStr returns a value as a string.  Unlike Evaluate, which can only
         /// return a floating point value, EvaluateStr can return a value of any data type.  Non-string values are converted to a string
         /// before being returned.
         /// </para></remarks> 
         STRING EvaluateStr() { RETURNSTRING(EvaluateStr_(ExprHandle));          }
         
         /// <summary> Rapidly evaluates an expression that was parsed with Parse or passed By Expression and returns a Pointer to the result</summary>
         /// <returns> Returns a pointer to the result instead of the result itself</returns>
         /// <remarks><para>Use this when you are interested in a pointer to the result instead of the result itself.  For instance, given an integer
         /// result, you might use a pointer in order to retrieve the hi byte and lo byte values.
         /// </para></remarks> 
         void * EvaluateVoid() { return  EvaluateVoid_(ExprHandle);          }
         
         /// <summary> Rapidly executes code that was parsed with Parse or passed By Expression for which a result is not required</summary>
         /// <remarks><para>This can be used in cases where the reutrn value is not needed, such as statements that represent code to be executed
         /// rather than a math expression.
         /// </para></remarks> 
         void Execute() { Execute_(ExprHandle);          }
         
         /// <summary> (Use Release instead)</summary>
         /// <remarks><para>Use Release</para></remarks> 
         void ReleaseExpr() { ReleaseExpr_(ExprHandle);          }
         
         /// <summary> Returns the handle of the parsed expression</summary>
         /// <remarks><para>This numeric value represents the pointer that uCalc uses internally for the ParsedExpr object.
         /// If you are iterating through a series of objects, such as with NextOverload, you may check Handle()
         /// to see when it becomes NULL (or 0) to know when to stop.</para></remarks>
         void *  Handle() { return  Handle_Overload_ParsedExpr_(ExprHandle);          }
         
         void Release() { ReleaseExpr(); ExprHandle = 0;          }
         /// <summary> Releases an expression that was parsed with Parse</summary>
         /// <remarks><para>Whenever an expression is parsed with Parse, the parsed expression remains in memory until released.
         /// When the parsed expression is no longer needed it should be released with Release to preserve memory.
         /// Note: An expression parsed with Parse must explicitely be released with Release unless the uCalc
         /// object it is in is released.  A parsed expression will not be released automatically when it simply
         /// goes out of scope.
         /// </para></remarks> 
         
         ParsedExpr() { ExprHandle = 0;          }
         ParsedExpr( uCalcPtr ExpressionHandle) { ExprHandle = ExpressionHandle;          }
         ParsedExpr(const ParsedExpr & ParsedExprInstance) { ExprHandle = ParsedExprInstance.ExprHandle;          }
      };
      
      /// <summary> Class for handling uCalc items</summary>
      struct Item {
         /// <summary> Handle for a uCalc.Item</summary>
         uCalcPtr ItemHandle;
         
         /// <summary> Returns the number of elements in a uCalc.Item</summary>
         /// <remarks><para>Depending on the kind of object, this may represent the number of elements in an array, the number of parameters in a function,
         /// the number of operands in an operator, the number of parameters in a syntax construct, the number of parts in a transform,
         /// the number of parts in a compound token, or the value explicitlely set in the Count property when calling Define.
         /// </para><para>
         /// When determining the number of elements, optional parameters in a function count as an element.  If the function
         /// has a variable number of arguments, then count is -1.
         /// </para><para>
         /// The value returned for a syntax construct may not seem obvious, especially if it has optional and/or alternative
         /// parts or some other complexity.
         /// </para><para>
         /// +++ the number of symbols in a uCalc
         /// instance,</para></remarks>
         int32_t ElementCount() { return  ElementCount_(ItemHandle);          }
         
         /// <summary> Returns the data type of a uCalc.Item</summary>
         /// <remarks><para>The returned data type object can be used to determine the type name, size, etc.</para></remarks>
         DataType GetDataType() { DataType TempObj(GetDataType_(ItemHandle)); return TempObj;          }
         
         /// <summary> Returns the callback address of a defined function (or operator)</summary>
         /// <remarks><para>The returned value is the actual pointer address of the function as used by uCalc internally, and not
         /// a delegate, even in .NET.  As such, in .NET this value can be used in the more general Define function but
         /// not functions like DefineFunction or DefineneOperator.</para></remarks>
         uCalcCallback FunctionAddress() { return  FunctionAddress_(ItemHandle);          }
         
         /// <summary> Determines if a uCalc.Item has the given properties</summary>
         /// <param name="ItemProperty"> The property you want to check for</param>
         /// <returns> Returns True if the uCalc.Item matches the given property, or False otherwise.</returns>
         /// <remarks><para>This helps you determine whether the uCalc.Item is a function, operator, variable, data type,
         /// literal value, etc, or has certain other special properties.  Note: Some items may have multiple
         /// properties.  For instance the Function property may be set to true for arrays.
         /// </para></remarks>
         bool IsProperty(ItemIsEnum ItemProperty) { return IsProperty_(ItemHandle,ItemProperty) != 0;          }
         
         /// <summary> Returns the name of the uCalc.Item</summary>
         /// <remarks><para>Items such as functions, operators, variables, etc generally are given a name when they are defined.
         /// That name can be retried with this function.</para></remarks>
         STRING  Name() { RETURNSTRING(Name_Overload_Item_(ItemHandle));          }
         
         /// <summary> Returns next uCalc.Item object that shares the same name</summary>
         /// <returns> Returns next uCalc.Item object that shares the same name, or a NULL (or 0) if there is none</returns>
         /// <remarks><para>If several functions, operators, parameters, etc are overloaded -- that is they are defined with different
         /// parameters but share the same name --, or if they (or variables) overshadow each other, this function returns
         /// the next uCalc.Item object that shares the same name.</para></remarks>
         Item NextOverload() { Item TempObj(NextOverload_(ItemHandle)); return TempObj;          }
         
         /// <summary> Returns the precedence level of the uCalc.Item</summary>
         /// <remarks><para>Operators, and some syntax constructs are defined with a precedence level.  When defining a new operator you
         /// may sometimes want to give it the same level as a pre-existing operator.</para></remarks> 
         int32_t  Precedence() { return  Precedence_Overload_Item_(ItemHandle);          }
         
         /// <summary> (Use Release instead)</summary>
         /// <remarks><para>Use Release</para></remarks> 
         void ReleaseItem() { ReleaseItem_(ItemHandle);          }
         
         /// <summary> Renames an item</summary>
         /// <param name="NewName"> The new name of the item</param>
         /// <remarks><para>
         /// Any defined item that has a name (such as a variable, constant, function, operator, etc) can be renamed with this function.
         /// </para></remarks> 
         void Rename( CONSTSTRING NewName ) { Rename_(ItemHandle,CNV(NewName));          }
         
         void Rename( CONSTCHAR NewName ) { Rename_(ItemHandle,NewName);          }
         
         /// <summary> Sets the value of a floating point variable +++ maybe SetValue instead of SetVariableValue</summary>
         /// <param name="Value"> Double precision floating point value that the variable will be set to</param>
         /// <remarks><para>When a floating point variable is defined with DefineVariable, you can later set a value for this variable using
         /// SetVariableValue.  Note: If your compiler supports pointers, you can define a uCalc variable that is directly attached
         /// to the address of a variable in your source code, and not have to use SetVariableValue.</para></remarks> 
         void SetVariableValue( double Value) { SetVariableValue_(ItemHandle,Value);          }
         
         /// <summary> Sets the value of a variable of any data type by passing a string</summary>
         /// <param name="Value"> String containing the value that will be converted and stored in the variable</param>
         /// <remarks><para>This lets you set the value of a variable defined with DefineVariable (or Define) regardless of the variable&#39;s data type.
         /// The value is passed as a string, and internally converted to the appropriate data type.  Note: Because a conversion takes
         /// place with SetVariableValueAny, it is typically slower than calling SetVariableValue for floating point, or one of the
         /// other versions of SetVariableValue specific to a data type.</para></remarks> 
         void SetVariableValueAny( CONSTSTRING Value ) { SetVariableValueAny_(ItemHandle,CNV(Value));          }
         
         void SetVariableValueAny( CONSTCHAR Value ) { SetVariableValueAny_(ItemHandle,Value);          }
         
         /// <summary> Sets the value of a Byte variable</summary>
         /// <param name="Value"> Byte value that the variable will be set to</param>
         /// <remarks><para>This lets you set the value of a Byte variable defined with DefineVariable (or Define)</para></remarks> 
         void SetVariableValueByte( uint8_t Value) { SetVariableValueByte_(ItemHandle,Value);          }
         
         /// <summary> Sets the value of a 16-bit integer variable</summary>
         /// <param name="Value"> 16-bit integer value that the variable will be set to</param>
         /// <remarks><para>This lets you set the value of a 16-bit integer variable defined with DefineVariable (or Define).  This works for
         /// both signed and unsigned 16-bit integers.</para></remarks> 
         void SetVariableValueInt16( int16_t Value) { SetVariableValueInt16_(ItemHandle,Value);          }
         
         /// <summary> Sets the value of a 32-bit integer variable</summary>
         /// <param name="Value"> 32-bit integer value that the variable will be set to</param>
         /// <remarks><para>This lets you set the value of a 32-bit integer variable defined with DefineVariable (or Define).  This works for
         /// both signed and unsigned 32-bit integers.</para></remarks> 
         void SetVariableValueInt32( int32_t Value) { SetVariableValueInt32_(ItemHandle,Value);          }
         
         /// <summary> Sets the value of a 64-bit integer variable</summary>
         /// <param name="Value"> 64-bit integer value that the variable will be set to</param>
         /// <remarks><para>This lets you set the value of a 64-bit integer variable defined with DefineVariable (or Define).  This works for
         /// both signed and unsigned 64-bit integers.</para></remarks> 
         void SetVariableValueInt64( int64_t Value) { SetVariableValueInt64_(ItemHandle,Value);          }
         
         /// <summary> Sets the value of a single precision floating point variable</summary>
         /// <param name="Value"> Single precision value that the variable will be set to</param>
         /// <remarks><para>This lets you set the value of a single precision variable defined with DefineVariable (or Define).</para></remarks> 
         void SetVariableValueSng( float Value) { SetVariableValueSng_(ItemHandle,Value);          }
         
         /// <summary> Sets the value of a double precision floating point variable</summary>
         /// <param name="Value"> Double precision floating point value that the variable will be set to</param>
         /// <remarks><para>SetVariableValueDbl is synonymous with SetVariableValue</para></remarks> 
         void SetVariableValueDbl( double Value) { SetVariableValueDbl_(ItemHandle,Value);          }
         
         /// <summary> Sets the value of a string variable</summary>
         /// <param name="Value"> String value that the variable will be set to</param>
         /// <remarks><para>This lets you set the value of a string variable defined with DefineVariable (or Define).</para></remarks> 
         void SetVariableValueStr( CONSTSTRING Value ) { SetVariableValueStr_(ItemHandle,CNV(Value));          }
         
         void SetVariableValueStr( CONSTCHAR Value ) { SetVariableValueStr_(ItemHandle,Value);          }
         
         /// <summary> Sets the value of a pointer variable</summary>
         /// <param name="Value"> Pointer that the variable will be set to</param>
         /// <remarks><para>This lets you set the value of a pointer variable defined with DefineVariable (or Define).  Note: This does not
         /// change the pointer of the variable.  It simply stores a pointer value in the variable.</para></remarks> 
         void SetVariableValuePtr( void * Value) { SetVariableValuePtr_(ItemHandle,Value);          }
         
         /// <summary> Sets the value of a variable to a value who&#39;s Pointer is passed</summary>
         /// <param name="Value"> Pointer to the value that the variable will be set to</param>
         /// <remarks><para>This lets you set the value of a variable by passing the pointer to the value you want to set the variable to.</para></remarks> 
         void SetVariableValueByPtr( void * Value) { SetVariableValueByPtr_(ItemHandle,Value);          }
         
         /// <summary> Returns the text data associated with a uCalc.Item</summary>
         /// <remarks><para>Certain items may have a text property that can be retrieved with a call to this function.  The literal text
         /// used to define an item is usually stored here.
         /// </para></remarks> 
         STRING Text() { RETURNSTRING(Text_(ItemHandle));          }
         
         /// <summary> Returns the uCalc object that the uCalc.Item belongs to</summary>
         /// <remarks><para>A uCalc.Item is defined in the context of a uCalc object.  You can later use this function to determine
         /// the uCalc object the defined item belongs to.</para></remarks> 
         uCalc  GetuCalc() { uCalc TempObj(GetuCalc_Overload_Item_(ItemHandle)); return TempObj;          }
         
         /// <summary> Returns the value (default double precision) associated with a uCalc.Item</summary>
         /// <remarks><para>Use this to retrieve the Double precision value associated with a variable.</para></remarks> 
         double Value() { return  Value_(ItemHandle);          }
         
         /// <summary> Returns the value (default double precision) associated with a uCalc.Item</summary>
         /// <remarks><para>Use this to retrieve the Byte value associated with a variable.</para></remarks> 
         uint8_t ValueByte() { return  ValueByte_(ItemHandle);          }
         
         /// <summary> Returns the 16-bit integer value associated with a uCalc.Item</summary>
         /// <remarks><para>Use this to retrieve the Int16 value associated with a variable.</para></remarks> 
         int16_t ValueInt16() { return  ValueInt16_(ItemHandle);          }
         
         /// <summary> Returns the 32-bit integer value associated with a uCalc.Item</summary>
         /// <remarks><para>Use this to retrieve the Int32 value associated with a variable.</para></remarks> 
         int32_t ValueInt32() { return  ValueInt32_(ItemHandle);          }
         
         /// <summary> Returns the 64-bit integer value associated with a uCalc.Item</summary>
         /// <remarks><para>Use this to retrieve the Int64 value associated with a variable.</para></remarks> 
         int64_t ValueInt64() { return  ValueInt64_(ItemHandle);          }
         
         /// <summary> Returns the Double precision value associated with a uCalc.Item</summary>
         /// <remarks><para>Use this to retrieve the Single precision value associated with a variable.</para></remarks> 
         float ValueSng() { return  ValueSng_(ItemHandle);          }
         
         /// <summary> Returns the Double precision value associated with a uCalc.Item</summary>
         /// <remarks><para>Synonymous with Value</para></remarks> 
         double ValueDbl() { return  ValueDbl_(ItemHandle);          }
         
         /// <summary> Returns the String value associated with a uCalc.Item</summary>
         /// <remarks><para>Use this to retrieve the String value associated with a variable.
         /// </para><para>
         /// ValueStr has a special property beyond the Value function for other data types in that it can return the value
         /// of an item regardless of what data type it has.  The value is returned as a string.
         /// </para></remarks> 
         STRING ValueStr() { RETURNSTRING(ValueStr_(ItemHandle));          }
         
         /// <summary> Returns the Pointer value associated with a uCalc.Item</summary>
         /// <remarks><para>Use this to retrieve the Pointer value associated with a variable.
         /// Note: If the uCalc.Item is a Variable of type Pointer, then ValuePtr returns a pointer value
         /// that is stored in that variable, whereas ValueAddr returns the Pointer of the variable itself.  In other words
         /// ValueAddr returns the address in memory of the current uCalc.Item value, while ValuePtr returns any value you may
         /// have set as the address the variable points to.</para></remarks> 
         void * ValuePtr() { return  ValuePtr_(ItemHandle);          }
         
         /// <summary> Returns the Pointer address of the value associated with a uCalc.Item</summary>
         /// <remarks><para>Use this to retrieve the Pointer address of the value associated with a variable (instead of
         /// the value itself).  Note: If the uCalc.Item is a Variable of type Pointer, then ValuePtr returns a pointer value
         /// that is stored in that variable, whereas ValueAddr returns the Pointer of the variable itself.  In other words
         /// ValueAddr returns the address in memory of the current uCalc.Item value, while ValuePtr returns any given address
         /// you may have stored as the value of the variable.</para></remarks> 
         void * ValueAddr() { return  ValueAddr_(ItemHandle);          }
         
         /// <summary> Returns the handle of the current uCalc.Item object</summary>
         void *  Handle() { return  Handle_Overload_Item_(ItemHandle);          }
         
         void Release() { ReleaseItem(); ItemHandle = 0;          }
         /// <summary> Releases an Item defined with Define or other related function</summary>
         /// <remarks><para>When a function, operator, variable, or other item is defined with such functions as
         /// Define, DefineFunction, DefineOperator, DefineVariable, these items remain in memory, until they are
         /// released with this function.  Items should generally be released when they are no longer needed, in order
         /// to preserve memory.
         /// </para><para>
         /// Note: A uCalc Item object must explicitely be released with Release unless the entire uCalc
         /// object it is a part of is is released.  An Item object will not be released automatically when it
         /// simply goes out of scope.
         /// </para></remarks> 
         
         Item() { ItemHandle = 0;          }
         Item( uCalcPtr PassedItemHandle) { ItemHandle = PassedItemHandle;          }
         Item(const Item & ItemHandleInstance) { ItemHandle = ItemHandleInstance.ItemHandle;          }
      };
      
      /// <summary> Class for retrieving arguments passed to a callback, returning a value, and more</summary>
      struct Callback {
         /// <summary> Handle of the expression part the current callback instance is refering to</summary>
         uCalcPtr ExprPartHandle;
         
         /// <summary> Returns the value of an argument passed to a uCalc.Callback function</summary>
         /// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         /// <remarks><para>When the name of a callback function is specified in DefineFunction or DefineOperator, that
         /// callback function will receive arguments uCalc passes to it through the Arg.  Arg handles values
         /// of the default Double floating point data type.  For other data types, ArgInt32, ArgStr, etc instead.
         /// </para><para>
         /// Note that Index is 1-based.  So if there are 3 arguments, the Index of the first will be
         /// 1 (not 0), and the second and third will be 2 and 3 respectively.
         /// </para><para>
         /// If you are defining a binary operator, 1 is the index for the left operand, and 2 for the right operand.
         /// For a unary operator, the index is 1.
         /// </para></remarks> 
         double Arg( int32_t Index) { return  Arg_(ExprPartHandle,Index);          }
         
         /// <summary> Returns the number of arguments being passed to a uCalc.Callback function</summary>
         /// <returns> 0 if no arg is passed; or a positive integer representing the number of args being passed.</returns>
         /// <remarks><para>This is useful for functions that have a variable number of arguments</para></remarks> 
         int32_t ArgCount() { return  ArgCount_(ExprPartHandle);          }
         
         /// <summary> Returns an object instance of the argument being passed to a uCalc.Callback function</summary>
         /// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         /// <returns> The returned object can be used (where relevent) just like any other uCalc.Item</returns>
         /// <remarks><para>Each argument is a uCalc.Item, just like anything else defined with Define.  With this object,
         /// you can do such things as determine the argument&#39;s data type, determine which instance of uCalc
         /// it belongs to, retrieve the pointer of the value of the argument, etc.</para></remarks> 
         Item ArgObj( int32_t Index) { Item TempObj(ArgObj_(ExprPartHandle,Index)); return TempObj;          }
         
         /// <summary> Returns the address of the argument value that is being passed</summary>
         /// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         /// <remarks><para>In addition to Arg which returns arguments that are of the default Double floating point numeric type, there
         /// are specialized functions for other datatypes such as ArgInt32, ArgInt64, etc.  However, if the argument is of
         /// a data type that does not have a specialized function, you can retrieve the address of the argument value as
         /// a pointer, and use that to retrieve the value.</para></remarks>  
         void * ArgAddr( int32_t Index) { return  ArgAddr_(ExprPartHandle,Index);          }
         
         /// <summary> Returns the object instance of the expression that&#39;s passed as an argument to a uCalc.Callback function+++Example</summary>
         /// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         /// <returns> Returns parsed expression object with which you can call functions like Evaluate or Execute</returns>
         /// <remarks><para>uCalc allows you to not only pass arguments by value or by reference.  But it also lets you pass an argument
         /// by expression.  When an expression is passed By Expression, the expresison is not evaluated before being passed.
         /// Instead a handle to the parsed expression is passed, allowing the callback to perform the evaluation.</para></remarks> 
         ParsedExpr ArgExpr( int32_t Index) { ParsedExpr TempObj(ArgExpr_(ExprPartHandle,Index)); return TempObj;          }
         
         /// <summary> Returns the Int32 integer argument passed to a uCalc.Callback function</summary>
         /// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         /// <remarks><para>If a function parameter (or operator operand) was explicitely defined as Int32, then the callback function can
         /// recieve it through ArgInt32.  If the argument is of a different but compatible numeric type, a conversion takes
         /// place first before being passed.</para></remarks> 
         int32_t ArgInt32( int32_t Index) { return  ArgInt32_(ExprPartHandle,Index);          }
         
         /// <summary> Returns the Int64 integer argument being passed to a uCalc.Callback function</summary>
         /// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         /// <remarks><para>If a function parameter (or operator operand) was explicitely defined as Int64, then the callback function can
         /// recieve it through ArgInt64.  If the argument is of a different but compatible numeric type, a conversion takes
         /// place first before being passed.</para></remarks> 
         int64_t ArgInt64( int32_t Index) { return  ArgInt64_(ExprPartHandle,Index);          }
         
         /// <summary> Returns a pointer value being passed as an argument to a uCalc.Callback function</summary>
         /// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         /// <remarks><para>If a function parameter (or operator operand) was explicitely defined as a Pointer, then the callback function can
         /// recieve it through ArgPtr.  Note that ArgPtr returns any address value that is being sent, whereas ArgAddr returns the
         /// address of the value being passed.</para></remarks> 
         void * ArgPtr( int32_t Index) { return  ArgPtr_(ExprPartHandle,Index);          }
         
         /// <summary> Returns the value being passed as argument in to a uCalc.Callback function in form of a string</summary>
         /// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         /// <remarks><para>In addition to returning an argument that was passed as a string, ArgStr has the special property of returning an argument that
         /// was passed as any data type.  If the argument was not a string, ArgStr converts it to a string before returning its value.</para></remarks> 
         STRING ArgStr( int32_t Index) { RETURNSTRING(ArgStr_(ExprPartHandle,Index));          }
         
         /// <summary> Sets the return value (default floating point) of a uCalc.Callback function</summary>
         /// <param name="Value"> Floating point value that your callback returns</param>
         /// <remarks><para>When you define a callback function with DefineFunction and you want to return a floating point number,
         /// do it with Return.
         /// </para><para>
         /// Note: For compilers in which Return is a reserved keyword, use ReturnDbl instead.
         /// </para></remarks> 
         void Return( double Value) { Return_(ExprPartHandle,Value);          }
         
         /// <summary> Sets the return value of a uCalc.Callback function as an Int32 integer</summary>
         /// <param name="Value"> Integer (32-bit) that your callback returns</param>
         /// <remarks><para>When you define a callback function with DefineFunction and you want to return a Int32,
         /// do it with ReturnInt32.</para></remarks> 
         void ReturnInt32( int32_t Value) { ReturnInt32_(ExprPartHandle,Value);          }
         
         /// <summary> Sets the return value of a uCalc.Callback function as an Int64 integer</summary>
         /// <param name="Value"> Integer (64-bit) that your callback returns</param>
         /// <remarks><para>When you define a callback function with DefineFunction and you want to return a Int64,
         /// do it with ReturnInt64.</para></remarks> 
         void ReturnInt64( int64_t Value) { ReturnInt64_(ExprPartHandle,Value);          }
         
         /// <summary> Sets the return value of a uCalc.Callback function as a Pointer</summary>
         /// <param name="Value"> Pointer value that your callback returns</param>
         /// <remarks><para>When you define a callback function with DefineFunction that returns a pointer value,
         /// do it with ReturnPtr.</para></remarks> 
         void ReturnPtr( void * Value) { ReturnPtr_(ExprPartHandle,Value);          }
         
         /// <summary> Sets the return value of a uCalc.Callback function as a String</summary>
         /// <param name="Value"> String value that your callback returns</param>
         /// <remarks><para>When you define a callback function with DefineFunction that returns a string,
         /// do it with ReturnStr.</para></remarks> 
         void ReturnStr( CONSTSTRING Value ) { ReturnStr_(ExprPartHandle,CNV(Value));          }
         
         void ReturnStr( CONSTCHAR Value ) { ReturnStr_(ExprPartHandle,Value);          }
         
         /// <summary> Raises an error in a uCalc.Callback function</summary>
         /// <param name="ErrNumber"> Index number of error to raise</param>
         /// <returns> Returns ErrorAbort, ErrorResume, or ErrorReRaise.</returns>
         /// <remarks><para>Your callback can raise an error by calling this funciton with one of the values of the
         /// ErrorNumberEnum enumerator.  End-user function definitions can also raise an error by calling Error().
         /// The return value is determined by error handlers that you may have added.  ErrorAbort (value of 0)
         /// is returned by default if there are no error handlers.</para></remarks>
         int32_t ErrorRaise( ErrorNumberEnum ErrNumber) { return  ErrorRaise_(ExprPartHandle,ErrNumber);          }
         
         /// <summary> Raises an error with a user-defined message in a uCalc.Callback function</summary>
         /// <param name="ErrMessage"> Text of custom error message to return</param>
         /// <returns> Returns ErrorAbort, ErrorResume, or ErrorReRaise.</returns>
         /// <remarks><para>Your callback can raise a custom error message (one not listed as a ErrorNumberEnum enumerator)
         /// by simply passing the text of the message to this function.</para></remarks>
         int32_t ErrorRaiseMessage( CONSTSTRING ErrMessage ) { return  ErrorRaiseMessage_(ExprPartHandle,CNV(ErrMessage));          }
         
         int32_t ErrorRaiseMessage( CONSTCHAR ErrMessage ) { return  ErrorRaiseMessage_(ExprPartHandle,ErrMessage);          }
         
         /// <summary> Returns the uCalc instance the current expression part belongs to</summary>
         /// <remarks><para>A callback is designed to work with all instances of uCalc.  However, at times you may need to know
         /// which particular instance of uCalc the expression part being passed belongs to.</para></remarks>
         uCalc  GetuCalc() { uCalc TempObj(GetuCalc_Overload_Callback_(ExprPartHandle)); return TempObj;          }
         
         /// <summary> Returns the handle of the sub expression passed to the callback</summary>
         /// <remarks><para>This pointer value is the argument received by a callback function associated with a function or operator
         /// defined with DefineFunction or DefineOperator.  Once a uCalc.Callback object is construct with this handle, this
         /// handle typically no longer needs to be refered to directly.  However, it&#39;s still avaible should you need it.
         /// uCalc interfaces with this handle underneath the hood of the uCalc.Callback object.
         /// </para></remarks>
         void *  Handle() { return  Handle_Overload_Callback_(ExprPartHandle);          }
         
         /// <summary> Returns the function or operator Item of the current expression part</summary>
         /// <remarks><para>If several functions and/or operators are mapped to the same callback function, you can use this to
         /// let you determine which one is currently being called, based on properties you can check.</para></remarks>
         Item OpItem() { Item TempObj(OpItem_(ExprPartHandle)); return TempObj;          }
         
         Callback() { ExprPartHandle = 0;          }
         Callback( uCalcPtr ExpressionHandle) { ExprPartHandle = ExpressionHandle;          }
         Callback(const Callback & ExprInstance) { ExprPartHandle = ExprInstance.ExprPartHandle;          }
      };
      
      /// <summary> Class for working with uCalc data types</summary>
      struct DataType {
         /// <summary> Handle for uCalc Data Type</summary>
         uCalcPtr DataTypeHandle;
         
         /// <summary> Allocates an array of the given data type</summary>
         /// <param name="ArraySize"> Size - in umber of elements - of the array you&#39;re allocating</param>
         /// <returns> Pointer of the array that was allocated</returns>
         /// <remarks><para>This allocates an array of the given type.  Elements of the array are set to the default value
         /// of that type.</para></remarks>
         void * AllocArray( int32_t ArraySize) { return  AllocArray_(DataTypeHandle,ArraySize);          }
         
         /// <summary> Allocates a scalar of the given type and copies data from the area indicated by the pointer</summary>
         /// <param name="ValuePtr"> Pointer to the data that is copied to the scalar being defined</param>
         /// <returns> Pointer of the scalar that was allocated</returns>
         void * AllocByPtr( void * ValuePtr) { return  AllocByPtr_(DataTypeHandle,ValuePtr);          }
         
         /// <summary> Allocates a scalar of the given data type</summary>
         /// <param name="Value"> Expression who&#39;s evaluated value will be copied to the allocated scalar</param>
         /// <returns> Pointer of the scalar that was allocated</returns>
         void * AllocScalar(  CONSTSTRING Value  = EMPTYSTR) { return  AllocScalar_(DataTypeHandle,CNV(Value));          }
         
         /// <summary> Returns the size (in bytes) of one unit of data for this data type</summary>
         int32_t ByteSize() { return  ByteSize_(DataTypeHandle);          }
         
         /// <summary> Returns the index of the data type</summary>
         /// <remarks><para>The return value is a BuiltInTypesEnum enumerator value that represents the data type of the
         /// current object.  For instance, if the data type is a string, the return value would be Type_String.
         /// </para></remarks>
         BuiltInTypesEnum Index() { return  Index_(DataTypeHandle);          }
         
         /// <summary> Returns the uCalc.Item handle of the data type</summary>
         /// <remarks><para>Each data type object can also be represented as a uCalc object by using this return value.</para></remarks>
         Item GetItem() { Item TempObj(GetItem_(DataTypeHandle)); return TempObj;          }
         
         /// <summary> Returns the name of the data type</summary>
         STRING  Name() { RETURNSTRING(Name_Overload_DataType_(DataTypeHandle));          }
         
         /// <summary> Releases an array that was allocated with AllocArray</summary>
         /// <param name="ValuePtr"> Pointer of uCalc array to be released from memory</param>
         /// <remarks><para>This should only be used for arrays that were explicitly allocated with uCalc&#39;s AllocArray.</para></remarks>
         void ReleaseArray( void * ValuePtr) { ReleaseArray_(DataTypeHandle,ValuePtr);          }
         
         /// <summary> Releases a scalar that was allocated with AllocScalar</summary>
         /// <param name="ValuePtr"> Pointer of uCalc scalar to be released from memory</param>
         /// <remarks><para>This should only be used for scalars that were explicitly allocated with uCalc&#39;s AllocScalar.</para></remarks>
         void ReleaseScalar( void * ValuePtr) { ReleaseScalar_(DataTypeHandle,ValuePtr);          }
         
         /// <summary> Resets the value of data that was allocated with SetScalar to the default value of the given data type</summary>
         /// <param name="ValuePtr"> Pointer of the uCalc array to be reset</param>
         /// <remarks><para>Resets all the elements of the array to the default value for the given data type.  This should only
         /// be used for arrays that were explicitly allocated with uCalc&#39;s AllocArray.</para></remarks>
         void ResetArray( void * ValuePtr) { ResetArray_(DataTypeHandle,ValuePtr);          }
         
         /// <summary> Resets the value of data that was allocated with SetScalar to the default value of the given data type</summary>
         /// <param name="ValuePtr"> Pointer of the uCalc scalar to be reset</param>
         void ResetScalar( void * ValuePtr) { ResetScalar_(DataTypeHandle,ValuePtr);          }
         
         /// <summary> Sets the value of an array at a given memory location that was allocated by AllocArray</summary>
         /// <param name="Destination"> Pointer of array (base address) who&#39;s element value is to be set</param>
         /// <param name="Source"> Pointer to a value of the same data type that will be copied to the element</param>
         /// <param name="Index"> Index of the array element who&#39;s value needs to be set</param>
         void SetArray( void * Destination ,  void * Source ,  int32_t Index) { SetArray_(DataTypeHandle,Destination , Source , Index);          }
         
         /// <summary> Sets the value of a scalar at a given memory location that was allocated by AllocScalar</summary>
         /// <param name="Destination"> Pointer of scalar who&#39;se value is to be set</param>
         /// <param name="Source"> Pointer to a value of the same data type that will be copied to the element</param>
         void SetScalar( void * Destination ,  void * Source) { SetScalar_(DataTypeHandle,Destination , Source);          }
         
         /// <summary> swaps the pointers of two variables that were allocated with AllocScalar or AllocArray</summary>
         /// <param name="ValuePtrA"> Pointer of variable 1</param>
         /// <param name="ValuePtrB"> Pointer of variable 2</param>
         void SwapScalarValues( void * ValuePtrA ,  void * ValuePtrB) { SwapScalarValues_(DataTypeHandle,ValuePtrA , ValuePtrB);          }
         
         /// <summary> Retuns a string representing the value at the given address formatted for the given data type</summary>
         /// <param name="ValuePtr"> Pointer of the data containing the value you want to display</param>
         /// <remarks><para>If you have a pointer to data of the given type, you can display this data as a string with this function.  Special formatting
         /// associated with a data type is used, if such formatting was defined.  For instance, if you&#39;re dealing with a Boolean, then
         /// you might get: True, Yes, On, etc, depending on how the formatting was configured with SetOutput.</para></remarks> 
         STRING  ToString( void * ValuePtr) { RETURNSTRING(ToString_Overload_DataType__ByPtr_(DataTypeHandle,ValuePtr));          }
         
         /// <summary> Retuns a formatted string of the given data type representing the value that was passed as text</summary>
         /// <param name="Value"> String representing data of any type that is to be represented as data of the given type</param>
         /// <remarks><para>If you pass a string, it will be converted to the given data type, and return using the formatting
         /// for that data type.</para></remarks>  
         STRING  ToString( CONSTSTRING Value ) { RETURNSTRING(ToString_Overload_DataType__ByStr_(DataTypeHandle,CNV(Value)));          }
         
         STRING  ToString( CONSTCHAR Value ) { RETURNSTRING(ToString_Overload_DataType__ByStr_(DataTypeHandle,Value));          }
         
         /// <summary> Returns the handle of the data type object</summary>
         /// <remarks><para>Generally for internal use</para></remarks>
         void *  Handle() { return  Handle_Overload_DataType_(DataTypeHandle);          }
         
         DataType() { DataTypeHandle = 0;          }
         DataType( uCalcPtr PassedDataTypeHandle) { DataTypeHandle = PassedDataTypeHandle;          }
         DataType(const DataType & DataTypeHandleInstance) { DataTypeHandle = DataTypeHandleInstance.DataTypeHandle;          }
      };
      
      /// <summary> uCalc transform class</summary>
      struct uCalcTransform {
         /// <summary> Handle for uCalc Transform instance</summary>
         uCalcPtr TransformHandle;
         /// <summary> Class for rules</summary>
         struct RuleClass {
            /// <summary> Handle for RuleClass</summary>
            uCalcPtr RuleHandle;
            
            /// <summary> Sets whether a rule is active</summary>
            /// <param name="TrueFalse"> Value that determines case is active</param>
            /// <returns> The current rule object</returns>
            /// <remarks><para>
            /// This sets whether a rule is active or not.
            /// When you no longer need a rule, you can release it entirely from memory with Release().  However, at times you may
            /// want to only temporarily deactivate a transform, and then later reactivate it for further use.  That is when you use
            /// this function.
            /// </para></remarks>
            RuleClass Active( bool TrueFalse) { RuleClass TempObj(Active_(RuleHandle,TrueFalse)); return TempObj;             }
            
            /// <summary> Sets Find part of a rule in a search</summary>
            /// <param name="Pattern"> Pattern for the Find part of a rule</param>
            /// <returns> The current rule object</returns>
            /// <remarks><para>
            /// This sets the Find part of a rule in a search.
            /// </para></remarks>
            RuleClass FindPart( CONSTSTRING Pattern ) { RuleClass TempObj(FindPart_(RuleHandle,CNV(Pattern))); return TempObj;             }
            
            RuleClass FindPart( CONSTCHAR Pattern ) { RuleClass TempObj(FindPart_(RuleHandle,Pattern)); return TempObj;             }
            
            /// <summary> Sets From part of a rule</summary>
            /// <param name="Pattern"> Pattern for the From part of a rule</param>
            /// <returns> The current rule object</returns>
            /// <remarks><para>
            /// This sets the From part of a rule.
            /// </para></remarks>
            RuleClass FromPart( CONSTSTRING Pattern ) { RuleClass TempObj(FromPart_(RuleHandle,CNV(Pattern))); return TempObj;             }
            
            RuleClass FromPart( CONSTCHAR Pattern ) { RuleClass TempObj(FromPart_(RuleHandle,Pattern)); return TempObj;             }
            
            /// <summary> Sets To part of a rule</summary>
            /// <param name="Pattern"> Pattern for the From part of a rule</param>
            /// <returns> The current rule object</returns>
            /// <remarks><para>
            /// This sets the To part of a rule.
            /// </para></remarks>
            RuleClass ToPart( CONSTSTRING Pattern ) { RuleClass TempObj(ToPart_(RuleHandle,CNV(Pattern))); return TempObj;             }
            
            RuleClass ToPart( CONSTCHAR Pattern ) { RuleClass TempObj(ToPart_(RuleHandle,Pattern)); return TempObj;             }
            
            /// <summary> Sets whether a rule is case-sensitive</summary>
            /// <param name="TrueFalse"> Value that determines case sensitivity</param>
            /// <returns> The current rule object</returns>
            /// <remarks><para>This sets whether a rule is case-sensitive or not.
            /// </para></remarks>
            RuleClass CaseSensitive( bool TrueFalse) { RuleClass TempObj(CaseSensitive_(RuleHandle,TrueFalse)); return TempObj;             }
            
            /// <summary> Sets whether a rule is whitespace-sensitive</summary>
            /// <param name="TrueFalse"> Value that determines whitespace sensitivity</param>
            /// <returns> The current rule object</returns>
            /// <remarks><para>This sets whether a rule is whitespace-sensitive or not.
            /// </para></remarks>
            RuleClass WhitespaceSensitive( bool TrueFalse) { RuleClass TempObj(WhitespaceSensitive_(RuleHandle,TrueFalse)); return TempObj;             }
            
            /// <summary> Sets code block sensitivity</summary>
            /// <param name="TrueFalse"> Value that determines code block sensitivity</param>
            /// <returns> The current rule object</returns>
            /// <remarks><para>This sets whether a rule is code block sensitive or not.  When set to True, a pattern will not match if it crosses the boundaries of
            /// a code block.
            /// </para><para>
            /// If set to False, then clode block delimiters are treated as ordinary generic tokens, with no special property as delimiters.
            /// </para><para>
            /// A code block [+++ for lack of better name] is a pattern enclosed between a starting token and an ending pattern.  Typically,
            /// things like parenthesis, brackets, and curly braces are defined as such.
            /// </para><para>
            /// So, if you define a pattern &quot;a { etc } b&quot;, it will match &quot;a x y z b c&quot;, &quot;a (x y z) b c&quot;, &quot;a (x) (y z) b c&quot;, but not &quot;a x y z (b c)&quot;,
            /// because the code block (b c) crosses the boundary of the &quot;a { etc } b&quot; pattern; the opening parenthesis is found in the would-be match,
            /// but the closing parenthesis is beyond it, whereas in the other examples, the both opening and closing parenthesis (when present) are
            /// fully within &quot;a { etc } b&quot;.
            /// </para><para>
            /// Note: You may use code block delimiter tokens in your patterns.
            /// </para></remarks>
            RuleClass CodeBlockSensitive( bool TrueFalse) { RuleClass TempObj(CodeBlockSensitive_(RuleHandle,TrueFalse)); return TempObj;             }
            
            /// <summary> Associates a comment with a rule</summary>
            /// <param name="CommentStr"> Text of the comment</param>
            /// <returns> The current rule object</returns>
            /// <remarks><para>
            /// Each rule can be assigned a comment.  This comment can later be used at your discretion, for instance in a help file system, or as a tool tip.
            /// </para></remarks>
            RuleClass  Comment( CONSTSTRING CommentStr ) { RuleClass TempObj(Comment_Overload_uCalcTransform_RuleClass_(RuleHandle,CNV(CommentStr))); return TempObj;             }
            
            RuleClass  Comment( CONSTCHAR CommentStr ) { RuleClass TempObj(Comment_Overload_uCalcTransform_RuleClass_(RuleHandle,CommentStr)); return TempObj;             }
            
            /// <summary> Returns a local transform object</summary>
            /// <returns> Local transform object</returns>
            /// <remarks><para>
            /// When a pattern is matched, you can zoom into the text and perform a local search just within the matched text, independantly
            /// from the rest of the text outside of the match.  Among various possible uses, it&#39;s ideal for parsing escape characters within quotes.
            /// Such characters are treated differently within the quote than outside the quote.
            /// </para></remarks>
            uCalcTransform LocalTransform() { uCalcTransform TempObj(LocalTransform_(RuleHandle)); return TempObj;             }
            
            /// <summary> Sets the soft maximum number of matches of this rule allowed</summary>
            /// <param name="Value"> Maximum number of matches of this rule allowed</param>
            /// <returns> The current rule object</returns>
            /// <remarks><para>
            /// This sets the soft maximum number of individual matches that this rule should find in order for any matches
            /// of this rule to count.  If more than this number of matches are found in the text, then none of the
            /// matches in this rule will count.  For instance if a rule searches for matches of &quot;a&quot; in &quot;a a r d v a r k&quot;,
            /// and the maximum is set to 2, then no matches of &quot;a&quot; will be counted at all, because there are more than
            /// 2 occurences of &quot;a&quot; in the text.  On the other hand, if Max is set to 3 or 4 or any larger value, then
            /// all the matches would count.
            /// </para><para>
            /// The default Maximum value is -1, which is the equivalent of no maximum limit.
            /// </para><para>
            /// If you want to stop counting after a given number of matches of a rule, without invalidating the matches
            /// that were already found, use StopAfter instead.
            /// </para></remarks> 
            RuleClass Maximum( int32_t Value) { RuleClass TempObj(Maximum_(RuleHandle,Value)); return TempObj;             }
            
            /// <summary> Sets the hard maximum number of matches of this rule allowed</summary>
            /// <param name="Value"> Maximum number of matches of this rule allowed</param>
            /// <returns> The current rule object</returns>
            /// <remarks><para>
            /// This sets the maximum number of individual matches that this rule should find in order for any matches
            /// of this rule to count.  If more than this number of matches are found in the text, then no matches from
            /// any rule will count (whereas in a soft maximum, only matches from the current rule will be cancelled).
            /// For instance if a rule searches for matches of &quot;a&quot; in &quot;a a r d v a r k&quot;,
            /// and the maximum is set to 2, then no matches of &quot;a&quot; will be counted at all, because there are more than
            /// 2 occurences of &quot;a&quot; in the text.  On the other hand, if Max is set to 3 or 4 or any larger value, then
            /// all the matches would count.
            /// </para><para>
            /// The default Maximum value is -1, which is the equivalent of no maximum limit.
            /// </para><para>
            /// If you want to stop counting after a given number of matches of a rule, without invalidating the matches
            /// that were already found, use StopAfter instead.
            /// </para></remarks> 
            RuleClass MaximumHard( int32_t Value) { RuleClass TempObj(MaximumHard_(RuleHandle,Value)); return TempObj;             }
            
            /// <summary> Sets the soft minimum number of matches that must be found in this rule</summary>
            /// <param name="Value"> Minimum number of matches required of this rule</param>
            /// <returns> The current rule object</returns>
            /// <remarks><para>
            /// This sets the soft minimum number of individual matches that this rule should find.
            /// </para><para>
            /// A soft minum (this one) differs from a hard minum (MinimumHard) in that if the soft minimum is not
            /// reached, then it is considered that there are no matches for this particular rule.  Other rules
            /// are not affected.  Whereas in a hard minimum, if this rule does not reach the required minimum,
            /// then the entire search or transform is invalidated, and it&#39;s considered that no match of any
            /// kind is found.
            /// </para><para>
            /// If less than this number of matches are found in the text, then none of the
            /// matches in this rule will count.  For instance if a rule searches for matches of &quot;a&quot; in &quot;a a r d v a r k&quot;,
            /// and the minimum is set to 5, then no matches of &quot;a&quot; will be counted at all, because there are not at least
            /// 5 occurences of &quot;a&quot; in the text.  On the other hand, if Min is set to 3 or less, then all the matches count.
            /// </para><para>
            /// Default: 0.
            /// </para></remarks> 
            RuleClass Minimum( int32_t Value) { RuleClass TempObj(Minimum_(RuleHandle,Value)); return TempObj;             }
            
            /// <summary> Sets the soft minimum number of matches that must be found in this rule</summary>
            /// <param name="Value"> Minimum number of matches required of this rule</param>
            /// <returns> The current rule object</returns>
            /// <remarks><para>
            /// This sets the soft minimum number of individual matches that this rule should find.
            /// </para><para>
            /// A soft minum (this one) differs from a hard minum (MinimumHard) in that if the soft minimum is not
            /// reached, then it is considered that there are no matches for this particular rule.  Other rules
            /// are not affected.  Whereas in a hard minimum, if this rule does not reach the required minimum,
            /// then the entire search or transform is invalidated, and it&#39;s considered that no match of any
            /// kind is found.
            /// </para><para>
            /// If less than this number of matches are found in the text, then none of the
            /// matches in this rule will count.  For instance if a rule searches for matches of &quot;a&quot; in &quot;a a r d v a r k&quot;,
            /// and the minimum is set to 5, then no matches of &quot;a&quot; will be counted at all, because there are not at least
            /// 5 occurences of &quot;a&quot; in the text.  On the other hand, if Min is set to 3 or less, then all the matches count.
            /// </para><para>
            /// Default: 0.
            /// </para></remarks> 
            RuleClass MinimumHard( int32_t Value) { RuleClass TempObj(MinimumHard_(RuleHandle,Value)); return TempObj;             }
            
            /// <summary> Sets the precedence level for a pattern</summary>
            /// <param name="Value"> Number of matches after which it should stop finding matches</param>
            /// <returns> The current rule object</returns>
            /// <remarks><para>
            /// Patterns that start (and/or end) with a pattern variable instead of a token or identifier can
            /// (and probably should) be assigned a precedence level.  You can choose any arbitrary numerical
            /// value.  The meaning of the precedence level depends on whether the value is greater, equal, or
            /// less than adjacent patterns with a precedence level.  It works similarly to precedence levels for
            /// operators.  For instance, if you define:
            /// </para><para>
            /// FromTo(&quot;{ a } + { b }&quot;, &quot;({ a } + { b })&quot;).Precedence(10);
            /// FromTo(&quot;{ a } * { b }&quot;, &quot;({ a } * { b })&quot;).Precedence(20);
            /// </para><para>
            /// Then
            /// </para><para>
            /// &quot;1 + 2 * 3 + 4&quot; Expands to &quot;(1 + (2 * 3) + 4)&quot;
            /// &quot;1 * 2 + 3 * 4&quot; Expands to &quot;(1 * 2) + (3 * 4)&quot;
            /// </para></remarks>
            RuleClass Precedence( int32_t Value) { RuleClass TempObj(Precedence_(RuleHandle,Value)); return TempObj;             }
            
            /// <summary> Sets quote sensitivity</summary>
            /// <param name="TrueFalse"> Value that determines quote sensitivity</param>
            /// <returns> The current rule object</returns>
            /// <remarks><para>This sets whether a rule is quote sensitive or not.
            /// Quote sensitivity is generally designed for use across all rules in a transform (or sub transform).  However,
            /// it can also be set in for the context of an individual pattern.  [One exception is within text that matches a pattern variable.+++]
            /// </para><para>
            /// When set to true, it means that a block of text within quotes is treated as one token.  As such patterns are not searched for
            /// within the quotes.  The contents of the quotes are skipped over.  You may still get around this and parse within quotes when
            /// necessary, by capturing the quote character itself, then parsing subsequent text.
            /// </para><para>
            /// The actual character(s) used as quotes are configurable with DefineToken.
            /// </para></remarks>
            RuleClass QuoteSensitive( bool TrueFalse) { RuleClass TempObj(QuoteSensitive_(RuleHandle,TrueFalse)); return TempObj;             }
            
            /// <summary> Returns the replacement text of a FromTo rule</summary>
            /// <returns> String representing the second argument that was used in FromTo</returns>
            /// <remarks><para>This allows you to keep track of the replacement text of any given rule, even after it has been defined.</para></remarks> 
            STRING ReplacementText() { RETURNSTRING(ReplacementText_(RuleHandle));             }
            
            /// <summary> Sets the number of matches this rule must find before matches start counting</summary>
            /// <param name="Value"> Number of matches that must be found before matches start counting</param>
            /// <returns> The current rule object</returns>
            /// <remarks><para>
            /// This sets a number of individual matches that this rule should find first.  These first few matches are
            /// not counted in the results.  Onlny matches starting after this number of initial matches are counted.
            /// If less than this number of matches are found in the text, no match will count.  For instance if a rule
            /// searches for matches of &quot;a&quot; in &quot;a a r d v a r k&quot;, starting after 2, then only the 3rd &quot;a&quot; (the one between
            /// &quot;v&quot; and &quot;r&quot;) will count.  If StartAfter is set to 3, there will be no matches.  If StartAfter is 1, then
            /// then the 2nd and 3rd occurences of &quot;a&quot; will count as matches.
            /// </para><para>
            /// The default StartAfter value is 0.  This means all matches that are found will count.
            /// </para></remarks> 
            RuleClass StartAfter( int32_t Value) { RuleClass TempObj(StartAfter_(RuleHandle,Value)); return TempObj;             }
            
            /// <summary> Sets the number of matches after which this rule should stop finding matches</summary>
            /// <param name="Value"> Number of matches after which it should stop finding matches</param>
            /// <returns> The current rule object</returns>
            /// <remarks><para>
            /// This sets a number of individual matches that this rule should find.  After this number of matches
            /// are found, all other matches for this rule are ignored.  For instance if a rule searches for matches
            /// of &quot;a&quot; in &quot;a a r d v a r k&quot;, and stops after 2, then only the first two letters here wil match; the
            /// third &quot;a&quot; (the one between &quot;v&quot; and &quot;r&quot;) will not count as a match.
            /// </para><para>
            /// The default StopAfter value is -1, which means it will not stop counting matches for this rule.
            /// </para><para>
            /// This function has some similarities with Max.  However, with Max, if the maximum value is not attained,
            /// none of the matches will count.
            /// </para></remarks> 
            RuleClass StopAfter( int32_t Value) { RuleClass TempObj(StopAfter_(RuleHandle,Value)); return TempObj;             }
            
            /// <summary> Determines whether a given match is skipped over</summary>
            /// <param name="TrueFalse"> Value that determines with match is skipped over</param>
            /// <returns> The current rule object</returns>
            /// <remarks><para>If SkipOver is set to true, then text that matches this pattern will be skipped over.
            /// This is useful for preventing uCalc from searching for matches of any other rules within
            /// certain text you want to skip over.
            /// </para><para>
            /// For instance if you skip over &quot;({ etc })&quot;, then everything in parenthesis will be skipped over.
            /// So if you want to find occurences of &quot;a&quot; in &quot;a b c (a b c a b c) a b c&quot;, it will find all occurences
            /// of &quot;a&quot;, except those within parenthesis, because everything in parenthesis is being skipped over.
            /// </para><para>
            /// Default: False
            /// </para></remarks> 
            RuleClass SkipOver( bool TrueFalse) { RuleClass TempObj(SkipOver_(RuleHandle,TrueFalse)); return TempObj;             }
            
            /// <summary> Determines whether to rewind back to the start of modified text for further parsing</summary>
            /// <param name="TrueFalse"> Value that determines if parser should rewind and parse again after a change</param>
            /// <returns> The current rule object</returns>
            /// <remarks><para>After text is modified with the replacement text of a rule, if this value is false (default),
            /// then uCalc will continue parsing from the position just after the insertion of the modified text.
            /// If this property is set to true, then the parser will rewind to the location where the match was
            /// found, and parse again, which will allow it to potentially find additional matches (of any rule,
            /// not just the current one) within the already modified text.
            /// </para><para>
            /// This property can be extremely useful in many situations.  However, special care should be taken
            /// so that an infinite reparsing session doesn&#39;t take place.  For instance, if a rule changes A to B,
            /// and another rule changes B to A, and this property is set ot true, and a Maximum value is not set,
            /// and there is an occurence of A, then A will change to B.  B will be parsed again and changed to A.
            /// The cycle will repeat itself indefinitely.
            /// </para><para>
            /// Default: False
            /// </para></remarks> 
            RuleClass RewindOnChange( bool TrueFalse) { RuleClass TempObj(RewindOnChange_(RuleHandle,TrueFalse)); return TempObj;             }
            
            /// <summary> Sets the maximum number of times a rule will cause a reparse at a given location</summary>
            /// <param name="TrueFalse"> Maximum number of times this rule can cause a re-parse</param>
            /// <param name="RaiseError"> Raises an error if the maximum is reached</param>
            /// <returns> The current rule object</returns>
            /// <remarks><para>
            /// [+++ Not completely thought out; might be removed]
            /// When ParseAgain is set to true, there is the potential for an infinite reparsing to take place.
            /// An obvious case is if a rule changes A to B, and another rule changes B to A.  However, there are
            /// less obvious cases where text might be modified in several ways before ending up in an infinite loop.
            /// </para><para>
            /// This property allows uCalc to break out of the cycle after text has been reparsed at a given location
            /// a certain number of times.
            /// </para><para>
            /// Once the maximum is passed, if RaiseError is set to true, an error indicating that the maximum was passed
            /// will be raised.  Otherwise, parsing will then silently continue from the end location of the modified text,
            /// as though ParseAgain were now false.
            /// </para></remarks> 
            RuleClass ParseAgainMax( bool TrueFalse ,  bool RaiseError) { RuleClass TempObj(ParseAgainMax_(RuleHandle,TrueFalse , RaiseError)); return TempObj;             }
            
            /// <summary> Sets whether a rule is case-sensitive</summary>
            /// <param name="TrueFalse"> Value that determines case sensitivity</param>
            /// <returns> The current rule object</returns>
            /// <remarks><para>This sets whether a rule is case sensitive or not.
            /// </para></remarks>
            RuleClass RightToLeft( bool TrueFalse) { RuleClass TempObj(RightToLeft_(RuleHandle,TrueFalse)); return TempObj;             }
            
            /// <summary> Releases a transform rule</summary>
            /// <remarks><para>When a transform rule is no longer needed it can be released in order to conserve memnory.
            /// </para></remarks>
            void ReleaseRule() { ReleaseRule_(RuleHandle);             }
            
            void Release() { ReleaseRule(); RuleHandle = 0;             }
            /// <summary> Releases a transform rule.  Transform rules are defined with FromTo.</summary>
            /// <remarks><para>
            /// This releases a transform rule from memory.
            /// </para></remarks> 
            
            RuleClass() { RuleHandle = 0;             }
            RuleClass( uCalcPtr rHandle) { RuleHandle = rHandle;             }
            RuleClass(const RuleClass & RuleInstance) { RuleHandle = RuleInstance.RuleHandle;             }
         };
         
         /// <summary> Class for collection of tokens</summary>
         struct Tokens {
            /// <summary> Handle for tokens</summary>
            uCalcPtr TokensHandle;
            
            /// <summary> Associates a comment with a token list</summary>
            /// <param name="CommentStr"> Text of the comment</param>
            /// <returns> Object containing list of tokens</returns>
            /// <remarks><para>Each token list can be assigned a comment.  This comment can later be used at your discretion, for instance in a help file system.
            /// </para></remarks>
            Tokens  Comment( CONSTSTRING CommentStr ) { Tokens TempObj(Comment_Overload_uCalcTransform_Tokens_(TokensHandle,CNV(CommentStr))); return TempObj;             }
            
            Tokens  Comment( CONSTCHAR CommentStr ) { Tokens TempObj(Comment_Overload_uCalcTransform_Tokens_(TokensHandle,CommentStr)); return TempObj;             }
            
            /// <summary> Imports tokens from another token list</summary>
            /// <param name="FromTokenList"> Token list to import +++ const Tokens &</param>
            /// <returns> Object containing list of tokens</returns>
            /// <remarks><para>
            /// Instead of defining a set of tokens each time, you may import a pre-existing token list.  This function imports from
            /// another token list object.  To import from a file, use Import (File).
            /// </para></remarks> 
            Tokens  Import( void * FromTokenList) { Tokens TempObj(Import_Overload_uCalcTransform_Tokens__Obj_(TokensHandle,FromTokenList)); return TempObj;             }
            
            /// <summary> Imports tokens from a token file</summary>
            /// <param name="TokenFile"> Token list file to import from</param>
            /// <returns> Object containing list of tokens</returns>
            /// <remarks><para>
            /// Instead of defining a set of tokens each time, you may import a pre-existing token list.  This function imports from
            /// a file.  To import from a token list object, use Import (Object).
            /// </para></remarks> 
            Tokens  Import( CONSTSTRING TokenFile ) { Tokens TempObj(Import_Overload_uCalcTransform_Tokens__File_(TokensHandle,CNV(TokenFile))); return TempObj;             }
            
            Tokens  Import( CONSTCHAR TokenFile ) { Tokens TempObj(Import_Overload_uCalcTransform_Tokens__File_(TokensHandle,TokenFile)); return TempObj;             }
            
            /// <summary> Defines a new token</summary>
            /// <param name="TokenDef"> Token definition</param>
            /// <param name="TokenType"> The type of token</param>
            /// <param name="EndMatch"> End match in a pair such as closing parenthesis or bracket</param>
            /// <param name="SubMatch"> Submatch</param>
            /// <param name="RegExGrammar"> Type of RegEx grammar</param>
            /// <returns> Token item object</returns>
            /// <remarks><para>
            /// uCalc comes with a default set of tokens.  These are general tokens for things like numbers alphanumeric words, whitespace, quotes,
            /// parenthesis, etc.  However you may define your own tokens to the exact specifications that you need.
            /// </para><para>
            /// A token is defined using a Regular Expression.
            /// </para><para>
            /// A token type can be any of those listed under TokenEnum (except for None).
            /// </para><para>
            /// Some tokens, such as parenthesis, brackets, or curly braces are typically defined as pairs.  So if &quot;(&quot; is defined
            /// as a token pair, then &quot;)&quot; would be the EndMatch.
            /// </para><para>
            /// In cases where you want a submatch within the token, and not the whole token, use the SubMatch argument with a numerical
            /// value represent the numerical index used in the regular expression.
            /// </para></remarks> 
            Item  Define( CONSTSTRING TokenDef  ,  TokenEnum TokenType  = Generic,  CONSTSTRING EndMatch  = EMPTYSTR,  int32_t SubMatch  = 0,  RegExGrammarEnum RegExGrammar  = RegEx_Default) { Item TempObj(Define_Overload_uCalcTransform_Tokens_(TokensHandle,CNV(TokenDef) ,TokenType,CNV(EndMatch),SubMatch,RegExGrammar)); return TempObj;             }
            
            Item  Define( CONSTCHAR TokenDef  ,  TokenEnum TokenType  = Generic,  CONSTCHAR EndMatch  = "",  int32_t SubMatch  = 0,  RegExGrammarEnum RegExGrammar  = RegEx_Default) { Item TempObj(Define_Overload_uCalcTransform_Tokens_(TokensHandle,TokenDef ,TokenType,EndMatch,SubMatch,RegExGrammar)); return TempObj;             }
            
            Tokens() { TokensHandle = 0;             }
            Tokens( uCalcPtr tHandle) { TokensHandle = tHandle;             }
            Tokens(const Tokens & TokensInstance) { TokensHandle = TokensInstance.TokensHandle;             }
         };
         
         /// <summary> Associates a comment with a transform</summary>
         /// <param name="CommentStr"> Text of the comment</param>
         /// <returns> Transform object</returns>
         /// <remarks><para>Each transform can be assigned a comment.  This comment can later be used at your discretion, for instance in a help file system.
         /// </para></remarks>
         uCalcTransform  Comment( CONSTSTRING CommentStr ) { uCalcTransform TempObj(Comment_Overload_uCalcTransform_(TransformHandle,CNV(CommentStr))); return TempObj;          }
         
         uCalcTransform  Comment( CONSTCHAR CommentStr ) { uCalcTransform TempObj(Comment_Overload_uCalcTransform_(TransformHandle,CommentStr)); return TempObj;          }
         
         /// <summary> Defines a rule (for search, transform, or filter)</summary>
         /// <param name="Comment"> Documentation/help/comment/tooltip type info associated with the rule. See Comment.</param>
         /// <param name="FindStr"> Pattern of text you want to match (The Find pattern in a search, or the From part in FromTo).</param>
         /// <param name="ToStr"> Replacement string (text that the From part should be changed to)</param>
         /// <param name="Min"> Soft minimum number of matches required.  See Min.</param>
         /// <param name="Max"> Soft maximum number of matches allowed.  See Max.</param>
         /// <param name="MinHard"> Hard minimum number of matches required. See MinHard</param>
         /// <param name="MaxHard"> Hard maximum number of matches allowed.  See MaxHard</param>
         /// <param name="Precedence"> Precedence level.  See Precedence.</param>
         /// <param name="StartAfter"> Number of matches to skip before matches start counting. See StartAfter.</param>
         /// <param name="StopAfter"> Number of matches to stop counting after.  See StopAfter.</param>
         /// <param name="SkipOver"> Makes it skip over text that is a match.  See SkipOver.</param>
         /// <param name="RewindOnChange"> Determines whether to rewind to the beginning of modified text and parse again</param>
         /// <param name="CaseSensitive"> Case sensitivity.  See CaseSensitive.</param>
         /// <param name="CodeblockSensitive"> Sets code block sensitivity.  See CodeblockSensitive.</param>
         /// <param name="QuoteSensitive"> Sets quote sensitivity.  See QuoteSensitive.</param>
         /// <param name="RightToLeft"> Right to left</param>
         /// <returns> Rule object that has just been defined</returns>
         /// <remarks><para>
         /// Defines a rule that contains a pattern used for finding text, along with a replacement string, and all the other properties.
         /// This function represents an alternative way of defining a rule, by supplying all the properties in this one function call.  These
         /// same properties can be called individually with separate functions.  Additional details for each property can be found in separate
         /// topics for function calls related to each property.
         /// </para><para>
         /// </para></remarks>
         RuleClass DefineRule( CONSTSTRING Comment  ,  CONSTSTRING FindStr  ,  CONSTSTRING ToStr  ,  int32_t Min ,  int32_t Max ,  int32_t MinHard ,  int32_t MaxHard ,  int32_t Precedence ,  int32_t StartAfter ,  int32_t StopAfter ,  bool SkipOver ,  bool RewindOnChange ,  bool CaseSensitive ,  bool CodeblockSensitive ,  bool QuoteSensitive ,  bool RightToLeft) { RuleClass TempObj(DefineRule_(TransformHandle,CNV(Comment) , CNV(FindStr) , CNV(ToStr) , Min , Max , MinHard , MaxHard , Precedence , StartAfter , StopAfter , SkipOver , RewindOnChange , CaseSensitive , CodeblockSensitive , QuoteSensitive , RightToLeft)); return TempObj;          }
         
         RuleClass DefineRule( CONSTCHAR Comment  ,  CONSTCHAR FindStr  ,  CONSTCHAR ToStr  ,  int32_t Min ,  int32_t Max ,  int32_t MinHard ,  int32_t MaxHard ,  int32_t Precedence ,  int32_t StartAfter ,  int32_t StopAfter ,  bool SkipOver ,  bool RewindOnChange ,  bool CaseSensitive ,  bool CodeblockSensitive ,  bool QuoteSensitive ,  bool RightToLeft) { RuleClass TempObj(DefineRule_(TransformHandle,Comment , FindStr , ToStr , Min , Max , MinHard , MaxHard , Precedence , StartAfter , StopAfter , SkipOver , RewindOnChange , CaseSensitive , CodeblockSensitive , QuoteSensitive , RightToLeft)); return TempObj;          }
         
         /// <summary> Defines a new token</summary>
         /// <param name="TokenDef"> Token definition</param>
         /// <param name="TokenType"> The type of token</param>
         /// <param name="EndMatch"> End match in a pair such as closing parenthesis or bracket</param>
         /// <param name="SubMatch"> Submatch</param>
         /// <param name="RegExGrammar"> Type of RegEx grammar</param>
         /// <returns> Token item</returns>
         /// <remarks><para>
         /// For details see Define (uCalcTransform)
         /// </para></remarks> 
         Item DefineToken( CONSTSTRING TokenDef  ,  TokenEnum TokenType  = Generic,  CONSTSTRING EndMatch  = EMPTYSTR,  int32_t SubMatch  = 0,  RegExGrammarEnum RegExGrammar  = RegEx_Default) { Item TempObj(DefineToken_(TransformHandle,CNV(TokenDef) ,TokenType,CNV(EndMatch),SubMatch,RegExGrammar)); return TempObj;          }
         
         Item DefineToken( CONSTCHAR TokenDef  ,  TokenEnum TokenType  = Generic,  CONSTCHAR EndMatch  = "",  int32_t SubMatch  = 0,  RegExGrammarEnum RegExGrammar  = RegEx_Default) { Item TempObj(DefineToken_(TransformHandle,TokenDef ,TokenType,EndMatch,SubMatch,RegExGrammar)); return TempObj;          }
         
         /// <summary> Returns the Tokens object for the current transform</summary>
         /// <returns> Token list object</returns>
         /// <remarks><para>
         /// Returns the Tokens object, which consists of a list of tokens for the current transform.
         /// You can use this to add, modify, delete, or reset the tokens.
         /// </para></remarks>
         Tokens GetTokens() { Tokens TempObj(GetTokens_(TransformHandle)); return TempObj;          }
         
         /// <summary> Defines a transform rule</summary>
         /// <param name="FromStr"> Pattern that describes the text to match</param>
         /// <param name="ToStr"> New text that the match should be changed to</param>
         /// <returns> Object for rule being defined</returns>
         /// <remarks><para>Whenever a piece of text matches the pattern described in FromStr, it is replaced with the text
         /// found in ToStr.
         /// </para></remarks> 
         RuleClass FromTo( CONSTSTRING FromStr  ,  CONSTSTRING ToStr  = EMPTYSTR) { RuleClass TempObj(FromTo_(TransformHandle,CNV(FromStr) ,CNV(ToStr))); return TempObj;          }
         
         RuleClass FromTo( CONSTCHAR FromStr  ,  CONSTCHAR ToStr  = "") { RuleClass TempObj(FromTo_(TransformHandle,FromStr ,ToStr)); return TempObj;          }
         
         /// <summary> Imports a transform from a file</summary>
         /// <param name="FileName"> Name of file containing transform to import</param>
         /// <returns> Transform object</returns>
         /// <remarks><para>
         /// This imports a transfrom from a file.  It can be a text file, in XML, uc, or ... format.
         /// </para></remarks>
         uCalcTransform  Import( CONSTSTRING FileName ) { uCalcTransform TempObj(Import_Overload_uCalcTransform_(TransformHandle,CNV(FileName))); return TempObj;          }
         
         uCalcTransform  Import( CONSTCHAR FileName ) { uCalcTransform TempObj(Import_Overload_uCalcTransform_(TransformHandle,FileName)); return TempObj;          }
         
         /// <summary> Sets the prefix that is inserted at the start of lists</summary>
         /// <param name="Prefix"> Prefix that will be inserted at the start of a list</param>
         /// <returns> Current transform object</returns>
         /// <remarks><para>
         /// Text that is returned as a list can have a prefix, postfix, and separators between elements.  A list of items
         /// within curly braces might have an open curly brace as prefix, a comma as delimiter, and a closing curly brace as postfix.
         /// </para></remarks>
         uCalcTransform ListPrefix( CONSTSTRING Prefix ) { uCalcTransform TempObj(ListPrefix_(TransformHandle,CNV(Prefix))); return TempObj;          }
         
         uCalcTransform ListPrefix( CONSTCHAR Prefix ) { uCalcTransform TempObj(ListPrefix_(TransformHandle,Prefix)); return TempObj;          }
         
         /// <summary> Sets the postfix that is inserted at the end of lists</summary>
         /// <param name="Postfix"> Postfix that will be inserted at the end of a list</param>
         /// <returns> Current transform object</returns>
         /// <remarks><para>
         /// Text that is returned as a list can have a prefix, postfix, and separators between elements.  A list of items
         /// within curly braces might have an open curly brace as prefix, a comma as delimiter, and a closing curly brace as postfix.
         /// </para></remarks>
         uCalcTransform ListPostfix( CONSTSTRING Postfix ) { uCalcTransform TempObj(ListPostfix_(TransformHandle,CNV(Postfix))); return TempObj;          }
         
         uCalcTransform ListPostfix( CONSTCHAR Postfix ) { uCalcTransform TempObj(ListPostfix_(TransformHandle,Postfix)); return TempObj;          }
         
         /// <summary> Sets the separator that will delimite the elements of a list</summary>
         /// <param name="Separator"> Separator that will delimite each element of a list</param>
         /// <returns> Current transform object</returns>
         /// <remarks><para>
         /// Text that is returned as a list can have a prefix, postfix, and separators between elements.  A list of items
         /// within curly braces might have an open curly brace as prefix, a comma as delimiter, and a closing curly brace as postfix.
         /// </para></remarks>
         uCalcTransform ListSeparator( CONSTSTRING Separator ) { uCalcTransform TempObj(ListSeparator_(TransformHandle,CNV(Separator))); return TempObj;          }
         
         uCalcTransform ListSeparator( CONSTCHAR Separator ) { uCalcTransform TempObj(ListSeparator_(TransformHandle,Separator)); return TempObj;          }
         
         /// <summary> Sets the separator that will delimite the elements of a list</summary>
         /// <param name="Prefix"> Prefix that will be inserted at the start of a list</param>
         /// <param name="Separator"> Separator that will delimite each element of a list</param>
         /// <param name="Postfix"> Postfix that will be inserted at the end of a list</param>
         /// <returns> Current transform object</returns>
         /// <remarks><para>
         /// Text that is returned as a list can have a prefix, postfix, and separators between elements.  A list of items
         /// within curly braces might have an open curly brace as prefix, a comma as delimiter, and a closing curly brace as postfix.
         /// </para></remarks>
         uCalcTransform ListDelimiters( CONSTSTRING Prefix  ,  CONSTSTRING Separator  ,  CONSTSTRING Postfix ) { uCalcTransform TempObj(ListDelimiters_(TransformHandle,CNV(Prefix) , CNV(Separator) , CNV(Postfix))); return TempObj;          }
         
         uCalcTransform ListDelimiters( CONSTCHAR Prefix  ,  CONSTCHAR Separator  ,  CONSTCHAR Postfix ) { uCalcTransform TempObj(ListDelimiters_(TransformHandle,Prefix , Separator , Postfix)); return TempObj;          }
         
         /// <summary> Sets and/or returns handle of math parser object that performs calculations related to the transform</summary>
         /// <param name="FMP"> Handle of a uCalc Fast Math Parser object  
         /// <returns> Math parser object associated with current transform</returns>
         /// <remarks><para>If your transform inovlves calcuations, then they will be performed using the
         /// uCalc Fast Math Parser object set with this function, or a default math parser
         /// object if this function is not called.  Each math parser can have its own sets of
         /// variables, functions, and other definitions.
         /// </para><para>
         /// If you pass the handle to an existing math parser, then that one will be used.  If nothing is passed to
         /// the function, then this function will merely return the existing math parser object.
         /// </para></remarks>
         uCalc MathParser(  void * FMP  = 0) { uCalc TempObj(MathParser_(TransformHandle,FMP)); return TempObj;          }
         
         /// <summary> Returns a new Pass object</summary>
         /// <param name="Share"> Share</param>
         /// <returns> Transform object</returns>
         /// <remarks><para>
         /// It is possible to create a transform with multiple passes.  Each pass can have a different set of rules.  The transform
         /// of each pass is executed in sequential order.  Passes can later be referenced later by index.  Calling NewPass the first
         /// time returns the same as Pass(0), the next call to NewPass refers to Pass(1), etc.
         /// </para></remarks> 
         uCalcTransform NewPass(  bool Share = false) { uCalcTransform TempObj(NewPass_(TransformHandle,Share)); return TempObj;          }
         
         /// <summary> Returns a new Pass object</summary>
         /// <param name="Index"> Index of pass to select</param>
         /// <param name="Share"> Share</param>
         /// <returns> Transform object</returns>
         /// <remarks><para>
         /// It is possible to create a transform with multiple passes.  Each pass can have a different set of rules.  Each pass
         /// is transformed in sequential order, starting with Pass(0).  A pass consists of a transform.
         /// </para></remarks> 
         uCalcTransform Pass( int32_t Index ,  bool Share = false) { uCalcTransform TempObj(Pass_(TransformHandle,Index ,Share)); return TempObj;          }
         
         /// <summary> Resets the transform object</summary>
         /// <param name="ResetOption"> Reset option</param>
         /// <returns> Current transform object</returns>
         /// <remarks><para>This clears the input string, transform rules, token definitions, Results.  It also resets properties to default settings
         /// </para></remarks>
         uCalcTransform Reset(  TransformResetEnum ResetOption  = Reset_All) { uCalcTransform TempObj(Reset_(TransformHandle,ResetOption)); return TempObj;          }
         
         /// <summary> Returns string result of transformed text</summary>
         /// <returns> String result of transformed text</returns>
         /// <remarks><para>After performing a text transformation, you may obtain the transformed result with this
         /// function
         /// </para></remarks>
         STRING ResultStr() { RETURNSTRING(ResultStr_(TransformHandle));          }
         
         /// <summary> Performs the transformation of text</summary>
         /// <returns> The current transform object</returns>
         /// <remarks><para>Once all the rules and parameters are set, you can call this function in order to
         /// perform the text transformation.
         /// </para><para>
         /// Prior to calling Transform, you must set the text to be transformed, with SetInput or SetInputByRef
         /// </para></remarks> 
         uCalcTransform Transform() { uCalcTransform TempObj(Transform_(TransformHandle)); return TempObj;          }
         
         /// <summary> Transforms an expression in one quick step</summary>
         /// <param name="Expression"> Expression to be expanded</param>
         /// <returns> String result of transform</returns>
         /// <remarks><para>
         /// [+++ Maybe rename Expand to something like Transform[Str]]
         /// When you would like to set text to be transformed, and transform the text, and return a result, all in one step,
         /// you can use this function, instead of calling SetInput, then Transform, then ResultStr.
         /// </para></remarks> 
         STRING Expand( CONSTSTRING Expression ) { RETURNSTRING(Expand_(TransformHandle,CNV(Expression)));          }
         
         STRING Expand( CONSTCHAR Expression ) { RETURNSTRING(Expand_(TransformHandle,Expression));          }
         
         /// <summary> Returns each step leading to an expression expansion</summary>
         /// <param name="Expression"> Expression to be expanded +++ See Expand_</param>
         /// <param name="Separator"> Separator used between steps</param>
         /// <param name="Prefix"> Prefix</param>
         /// <param name="Postfix"> Postfix</param>
         /// <returns> String containg steps leading up to transform result</returns>
         /// <remarks><para>
         /// Transforms an expression and shows intermediate steps
         /// </para></remarks>
         STRING ExpandWithSteps( CONSTSTRING Expression  ,  CONSTSTRING Separator  = EMPTYSTR,  CONSTSTRING Prefix  = EMPTYSTR,  CONSTSTRING Postfix  = EMPTYSTR) { RETURNSTRING(ExpandWithSteps_(TransformHandle,CNV(Expression) ,CNV(Separator),CNV(Prefix),CNV(Postfix)));          }
         
         STRING ExpandWithSteps( CONSTCHAR Expression  ,  CONSTCHAR Separator  = "",  CONSTCHAR Prefix  = "",  CONSTCHAR Postfix  = "") { RETURNSTRING(ExpandWithSteps_(TransformHandle,Expression ,Separator,Prefix,Postfix));          }
         
         /// <summary> Sets the text input that is to be transformed</summary>
         /// <param name="InputStr"> Text that mast be parsed and modified</param>
         /// <returns> Current transform object</returns>
         /// <remarks><para>The text that you want to transform should be passed to this function.
         /// </para></remarks>
         uCalcTransform SetInput( CONSTSTRING InputStr ) { uCalcTransform TempObj(SetInput_(TransformHandle,CNV(InputStr))); return TempObj;          }
         
         uCalcTransform SetInput( CONSTCHAR InputStr ) { uCalcTransform TempObj(SetInput_(TransformHandle,InputStr)); return TempObj;          }
         
         /// <summary> Sets the string variable containing text that is to be transformed or searched through</summary>
         /// <param name="InputStr"> Variable containing text thta must be parsed and modified</param>
         /// <returns> Current transform object</returns>
         /// <remarks><para>The text that you want to transform or search through should be passed to this function.
         /// This function is similar to Input, except that instead of copying a string, this passes a
         /// string variable by reference.  This is more speed and memory efficient when dealing with large
         /// amounts of text and/or situations where text is being parsed repeatedly.
         /// </para></remarks> 
         uCalcTransform SetInputByRef( void * InputStr) { uCalcTransform TempObj(SetInputByRef_(TransformHandle,InputStr)); return TempObj;          }
         
         /// <summary> Sets the math parser object by handle</summary>
         /// <param name="FMP"> Handle of uCalc object +++ const uCalc & FMP</param>
         /// <returns> Current transform object</returns>
         /// <remarks><para>
         /// This sets the math parser object that is used when expression evaluations are involved in transforming text.
         /// For instance when { @Eval: ... } is used in the pattern or ~Eval() in the text.
         /// </para></remarks> 
         uCalcTransform SetParser( void * FMP) { uCalcTransform TempObj(SetParser_(TransformHandle,FMP)); return TempObj;          }
         
         /// <summary> Sets properties for a transform</summary>
         /// <param name="TransformProperties"> Properties for transforms</param>
         /// <returns> Current transform object</returns>
         /// <remarks><para>
         /// This sets the properties for a transform.  A transform can keep track of each change, which can
         /// </para><para>
         /// These properties are all off by default, as they add a small speed
         /// penalty and take up memory to store each change, depending on the option.
         /// </para><para>
         /// To set multiple properties, add them up, or use the OR operator (|| in some programming languages).
         /// </para></remarks>
         uCalcTransform SetProperties(  TransformPropertiesEnum TransformProperties  = NoTransformProperties) { uCalcTransform TempObj(SetProperties_(TransformHandle,TransformProperties)); return TempObj;          }
         
         /// <summary> Indicates whether text was modified in the most recent Transform operation</summary>
         /// <returns> True if text was modified in the most recent transformation operation or false otherwise</returns>
         /// <remarks><para>
         /// If text was modified after a call to Transform, then this function returns True.  When transforming large amounts of text,
         /// transferring strings back and forth with ReturnStr() can be expensive.  Checking this property allows you to skip unnecessary
         /// steps when text has not been modified.
         /// </para></remarks> 
         bool WasModified() { return WasModified_(TransformHandle) != 0;          }
         
         /// <summary> Releases a transform</summary>
         /// <remarks><para>When a transform is no longer needed it can be released in order to conserve memnory.
         /// </para></remarks>
         void ReleaseTransform() { ReleaseTransform_(TransformHandle);          }
         
         /// <summary> Returns a rule object from a transform</summary>
         /// <param name="Index"> Index of rule</param>
         /// <returns> Rule object</returns>
         /// <remarks><para>
         /// The default rule is located at index -1.  All subsequent rules inherent properties from the default rule.
         /// </para><para>
         /// If no argument is passed, then RuleClass returns the next empty new rule in the list.
         /// </para></remarks>
         RuleClass GetRule(  int32_t Index  = 0) { RuleClass TempObj(GetRule_(TransformHandle,Index)); return TempObj;          }
         
         /// <summary> Returns the defaults settings for rules</summary>
         /// <returns> Default rule object</returns>
         /// <remarks><para>
         /// Each FromTo or Find rule has many configurable properties that can be set.  Instead of excplicitely setting all of these many properties
         /// with each rule definition, you may define default settings that will automatically be assigned to new rules.  Any of these default settings
         /// can be overridden by explicitely setting the desired property for a given rule.
         /// </para></remarks> 
         RuleClass RulesDefault() { RuleClass TempObj(RulesDefault_(TransformHandle)); return TempObj;          }
         
         /// <summary> Returns the overall settings for a transform</summary>
         /// <returns> Overall rule object</returns>
         /// <remarks><para>
         /// This returns an object containing the properties for a transform or search object.  This object is a Rules object, and conatains the
         /// same properties listed for individual rules, but works in a slightly different context.  For instance, if you set the StopAfter property
         /// of FromTo(&quot;a&quot;, &quot;b&quot;) to 5, then it will only change the first five occurences of &quot;a&quot; to &quot;b&quot;, whereas if you set the RulesOverall StopAfter
         /// property to 5, and you have rules FromTo(&quot;a&quot;, &quot;b&quot;) and FromTo(&quot;x&quot;, &quot;y&quot;) then it will stop changing any occurence of either rule, after the
         /// combined total of changes reaches 5; &quot;a&quot; might change to &quot;b&quot; 2 times and &quot;x&quot; to &quot;y&quot; 3 times, or whichever combination does not total more
         /// than 5.  You might set QuoteSensitive to &quot;true&quot; overall, while you may set it to false within a given rule.  In certain combinations,
         /// overall rules may clash with individual rules.  Although all combinations are allowed, not all combinations make sense.  When setting
         /// the case sensitivity property of individual rules differently from the overall property, keep in mind that if the overall case sensitivity
         /// is false, then all symbols are stored in the symbol table in all upper case.
         /// </para></remarks> 
         RuleClass RulesOverall() { RuleClass TempObj(RulesOverall_(TransformHandle)); return TempObj;          }
         
         void Release() { ReleaseTransform(); TransformHandle = 0;          }
         /// <summary> Releases a transform</summary>
         /// <remarks><para>
         /// This releases a transform from memory.
         /// </para></remarks> 
         
         uCalcTransform() {           }
         uCalcTransform( uCalcPtr tHandle) { TransformHandle = tHandle;          }
         uCalcTransform(const uCalcTransform & TransformInstance) { TransformHandle = TransformInstance.TransformHandle;          }
      }; // uCalcTransform
      
      /// <summary> Adds an error handler</summary>
      /// <param name="ErrorHandlerAddress"> Address of the callback function containing the error handler code</param>
      /// <param name="Position"> Position in which to insert the error handler</param>
      /// <returns> The error handler can later be removed using this return object</returns>
      /// <remarks><para>Use this to set a a callback function that is called whenever there is an error.  Your callback can
      /// do things like alert the user of the error, correct the error and resume, keep a log of errors, etc.
      /// </para><para>
      /// By default, if there are multiple error handlers, they are called starting with the most recently
      /// defined one first, ending with the first one to be defined last.  In order to define a handler in a different
      /// order, specify the Position value of where it should be inserted; 0 so that it is called first, -1 so it
      /// is called last, or a positive integer indicated where it should be inserted.  If the Postion value is out
      /// of bounds, it is adjusted into the maximum acceptable value.
      /// </para><para>
      /// If you set ErrorResponse to ErrorHandlerResponseEnum.ErrorResume, subsequent error handler callbacks will
      /// be bypassed and uCalc will resume operations as though no error had taken place.  Note that the condition
      /// that caused the error must be corrected before resuming, otherwise uCalc may get stuck in an infinite loop.
      /// </para><para>
      /// Some functions, such as ErrorExpression, ErrorLocation, and ErrorSymbol produce a meaningful result only
      /// during the parsing stage.  They do not cause other problems beyond meaningless data during the evaluation stage.
      /// </para><para>
      /// For Floating Point Environment errors, such as: Err_Float_Division_By_Zero, Err_Float_Inexact_Result,
      /// Err_Float_Invalid_Operation, Err_Float_Overflow, and Err_Float_Underflow, the uCalc error handler callbacks
      /// are called only if the corresponding environment bits are not masked.
      /// </para><para>
      /// </para></remarks> 
      Item AddErrorHandler( UCCALLBACK ErrorHandlerAddress ,  int32_t Position  = 0) { Item TempObj(AddErrorHandler_(uCalcHandle,ErrorHandlerAddress ,Position)); return TempObj;       }
      
      /// <summary> Creates a new uCalc instance</summary>
      /// <remarks><para>This is not meant to be called directly.</para></remarks>
      uCalcPtr CreateuCalc() { return  CreateuCalc_(uCalcHandle);       }
      
      /// <summary> Returns a data type object</summary>
      /// <param name="Index"> Index associated with the data type</param>
      /// <remarks><para>Each instance of uCalc has its own separate handle &amp; data type object for any given data type.  For
      /// instance, GetItemOf(&quot;Int64&quot;) in one instance of uCalc does not share the same handle as GetItemOf(&quot;Int64&quot;) in
      /// another instance.  This allows the same data type to have a different name and different formatting in
      /// different instances.  However, they share the same data type index across instances.  Given the index
      /// of a data type, you can directly obtain the proper data type object for a particular uCalc instance,
      /// faster than obtaining it by name as text with GetItemOf.</para></remarks>
      DataType DataTypeOfIndex( BuiltInTypesEnum Index) { DataType TempObj(DataTypeOfIndex_(uCalcHandle,Index)); return TempObj;       }
      
      /// <summary> Defines a uCalc.Item object (a constunction, function, operator, variable, etc)</summary>
      /// <param name="Expression"> Text containing item to be defined.  Expression must start with a keyword such as Function, Operator, Variable, Token, Syntax, etc., followed by a colon, before the expression</param>
      /// <param name="Address"> Address a function or variable is attached to</param>
      /// <param name="ExistingItem"> Handle of existing item this definition is associated with</param>
      /// <remarks><para>Functions, operators, variables, constants, tokens, syntax constructs, etc., can be defined with
      /// this method.  Although there are specialized methods that do the same things, such as DefineFunction,
      /// DefineOperator, DefineVariable, and DefineConstant, this method may in same cases offer
      /// more flexibility.
      /// </para><para>
      /// The expression must be preceded by a keyword and colon, such as &quot;Variable:&quot;.  For a full list of keywords,
      /// see Definition Keywords.
      /// </para><para>
      /// Sections are separated by ~~ (two tilda signs).
      /// </para></remarks> 
      Item  Define( CONSTSTRING Expression  ,  void * Address  = 0,  void * ExistingItem  = 0) { Item TempObj(Define_Overload_uCalc_(uCalcHandle,CNV(Expression) ,Address,ExistingItem)); return TempObj;       }
      
      Item  Define( CONSTCHAR Expression  ,  void * Address  = 0,  void * ExistingItem  = 0) { Item TempObj(Define_Overload_uCalc_(uCalcHandle,Expression ,Address,ExistingItem)); return TempObj;       }
      
      /// <summary> Defines a constant</summary>
      /// <param name="Definition"> Text containing constant to be defined</param>
      /// <param name="VariableAddress"> Address the constant is attached to</param>
      /// <remarks><para>
      /// A constant definition is similar to that of a variable defined with DefineVariable, except that the end-user is not allowed to modify the value of a constant.
      /// You are, however, allowed to perform lower-level operations such as renaming a constant, or deleting it and re-defining a new constant or variable with the same
      /// name but different value.  If you attach the constant to the address of a variable in your host program, the host program may change the value.  However, when
      /// the end-user attempts to assign a value to the constant, an error is raised.
      /// </para><para>
      /// DefineConstant(&quot;x&quot;) is the same as: Define(&quot;Lock ~~ Variable: x&quot;)
      /// </para></remarks> 
      Item DefineConstant( CONSTSTRING Definition  ,  void * VariableAddress  = 0) { Item TempObj(DefineConstant_(uCalcHandle,CNV(Definition) ,VariableAddress)); return TempObj;       }
      
      Item DefineConstant( CONSTCHAR Definition  ,  void * VariableAddress  = 0) { Item TempObj(DefineConstant_(uCalcHandle,Definition ,VariableAddress)); return TempObj;       }
      
      /// <summary> Defines a data type</summary>
      /// <param name="TypeSize"> Number of bytes 1 scalar value of this type takes</param>
      /// <param name="AllocArrayValue"> Address of a function that will allocate an array of this type</param>
      /// <param name="AllocScalar"> Address of a function that will allocate a scalar of this type.  If a string is passed, this function must convert it to this type and store the value, otherwise the function must store a default value for this type.</param>
      /// <param name="AllocScalarByPtr"> Address of a function that takes a pointer to a value of this type, allocates a scalar and stores the copied value in the allocated space.</param>
      /// <param name="DeleteArray"> Address of a function that frees a memory block that was allocated with AllocValueArray</param>
      /// <param name="DeleteScalar"> Address of a function that frees a memory block that was allocated by AllocScalar or AllocScalarByPtr.</param>
      /// <param name="ResetScalar"> Address of a function that resets an allocated scalar back to the default value of this type.</param>
      /// <param name="ResetArray"> Address of a function that resets array elements back to the default value of this type.</param>
      /// <param name="SetValueArray"> Address of a function that stores an array element value.</param>
      /// <param name="SetValueScalar"> Address of a function that stores a scalar value.</param>
      /// <param name="SwapValues"> Address of a function that swaps two values of this type.</param>
      /// <param name="ToString"> Address of a function that converts this data type to a string.</param>
      /// <remarks><para>
      /// This allows you to define a data type that functions at the same level as the intrinsic types that come with uCalc.
      /// </para></remarks> 
      Item DefineDataType( int32_t TypeSize ,  void * AllocArrayValue ,  void * AllocScalar ,  void * AllocScalarByPtr ,  void * DeleteArray ,  void * DeleteScalar ,  void * ResetScalar ,  void * ResetArray ,  void * SetValueArray ,  void * SetValueScalar ,  void * SwapValues ,  void * ToString) { Item TempObj(DefineDataType_(uCalcHandle,TypeSize , AllocArrayValue , AllocScalar , AllocScalarByPtr , DeleteArray , DeleteScalar , ResetScalar , ResetArray , SetValueArray , SetValueScalar , SwapValues , ToString)); return TempObj;       }
      
      /// <summary> Defines a function</summary>
      /// <param name="Definition"> Text containing function to be defined</param>
      /// <param name="FunctionAddress"> Address of callback function this function is attached to</param>
      /// <remarks><para>
      /// A function definition can either be self-contained, or it may be attached to the address of a callback function in your
      /// source code.
      /// </para><para>
      /// DefineFunction(&quot;f(x) = ...&quot;) is the same as: Define(&quot;Function: ...&quot;)
      /// </para></remarks> 
      Item DefineFunction( CONSTSTRING Definition  ,  UCCALLBACK FunctionAddress  = 0) { Item TempObj(DefineFunction_(uCalcHandle,CNV(Definition) ,FunctionAddress)); return TempObj;       }
      
      Item DefineFunction( CONSTCHAR Definition  ,  UCCALLBACK FunctionAddress  = 0) { Item TempObj(DefineFunction_(uCalcHandle,Definition ,FunctionAddress)); return TempObj;       }
      
      /// <summary> Defines an operator</summary>
      /// <param name="Definition"> Text containing operator to be defined</param>
      /// <param name="iPrecedence"> Precedence level of the operator.</param>
      /// <param name="Grouping"> Operator associativity; arithmetic operators are typically grouped left-to-right; special operators, such as the equal sign, =, assignment operator are right-to-left</param>
      /// <param name="FunctionAddress"> Address of callback function this operator is attached to</param>
      /// <remarks><para>
      /// Many details for defining an operator are similar to those of defining a function.  In additionn, all operators must be
      /// defined with a precedence level.  You may use the Precedence function to obtain precedence levels of existing operators,
      /// and select a precedence level in comparison to those operators.  By default operators are grouped left to right.  But you
      /// may also set it to right to left.  This means that when adjacent operators have the same precedence level, the operands on
      /// the right will be evaluated first.
      /// </para><para>
      /// DefineOperator(&quot;...&quot;) is the same as: Define(&quot;Operator: ...&quot;)
      /// More later+++
      /// </para></remarks> 
      Item DefineOperator( CONSTSTRING Definition  ,  int32_t iPrecedence ,  GroupingEnum Grouping  = GroupLeftToRight,  UCCALLBACK FunctionAddress  = 0) { Item TempObj(DefineOperator_(uCalcHandle,CNV(Definition) , iPrecedence ,Grouping,FunctionAddress)); return TempObj;       }
      
      Item DefineOperator( CONSTCHAR Definition  ,  int32_t iPrecedence ,  GroupingEnum Grouping  = GroupLeftToRight,  UCCALLBACK FunctionAddress  = 0) { Item TempObj(DefineOperator_(uCalcHandle,Definition , iPrecedence ,Grouping,FunctionAddress)); return TempObj;       }
      
      /// <summary> Defines a variable</summary>
      /// <param name="Definition"> Text containing variable to be defined</param>
      /// <param name="VariableAddress"> Address of variable in host program this uCalc variable is attached to</param>
      /// <remarks><para>
      /// A variable consists of ...
      /// </para><para>
      /// DefineVariable(&quot;...&quot;) is the same as: Define(&quot;Variable: x&quot;)
      /// </para></remarks> 
      Item DefineVariable( CONSTSTRING Definition  ,  void * VariableAddress  = 0) { Item TempObj(DefineVariable_(uCalcHandle,CNV(Definition) ,VariableAddress)); return TempObj;       }
      
      Item DefineVariable( CONSTCHAR Definition  ,  void * VariableAddress  = 0) { Item TempObj(DefineVariable_(uCalcHandle,Definition ,VariableAddress)); return TempObj;       }
      
      /// <summary> Returns the expression associated with an error that was raised</summary>
      STRING ErrorExpression() { RETURNSTRING(ErrorExpression_(uCalcHandle));       }
      
      /// <summary> Returns the location of an error that was raised</summary>
      int32_t ErrorLocation() { return  ErrorLocation_(uCalcHandle);       }
      
      /// <summary> Returns a message associated with the error that was raised</summary>
      /// <param name="Index"> Index of error message to return</param>
      /// <remarks><para>If an error was raised, this returns a message describing the recent error.  If the Index argument is specified,
      /// then the error message associated with this Index is returned.  A value must be chosen from ErrorNumberEnum.
      /// </para><para>
      /// Note: Errors are cleared whenever a new expression is parsed or a new item is defined.
      /// </para></remarks> 
      STRING ErrorMessage(  ErrorNumberEnum Index  = Err_None) { RETURNSTRING(ErrorMessage_(uCalcHandle,Index));       }
      
      /// <summary> Returns the error number of the error that was raised</summary>
      /// <remarks><para>
      /// Returns the error number of the most recently raised error.
      /// </para><para>
      /// Note: Errors are cleared whenever a new expression is parsed or a new item is defined.
      /// </para></remarks>
      ErrorNumberEnum ErrorNumber() { return  ErrorNumber_(uCalcHandle);       }
      
      /// <summary> Sets the error response for an error handler</summary>
      /// <param name="Response"> Indicates what should happen now that an error is raised</param>
      /// <remarks><para>When an error is raised, you have the option of aborting the rest of the option (which is the default option),
      /// or you may correct the problem, and then resume the operation.  Or you may perform an action, and than raise the
      /// error again for a subsequent error handle to take care of.
      /// </para><para>
      /// This method can only be called from within a callback of an error handler routine definined with AddErrorHandler.
      /// The options to use can be found at ErrorHandlerResponseEnum.
      /// </para></remarks> 
      void ErrorResponse( ErrorHandlerResponseEnum Response) { ErrorResponse_(uCalcHandle,Response);       }
      
      /// <summary> Returns the symbol associated with the error that is raised</summary>
      STRING ErrorSymbol() { RETURNSTRING(ErrorSymbol_(uCalcHandle));       }
      
      /// <summary> Sets a new expression after an error is raised</summary>
      /// <param name="Expression"> Error expression</param>
      void ErrorSetExpression( CONSTSTRING Expression ) { ErrorSetExpression_(uCalcHandle,CNV(Expression));       }
      
      void ErrorSetExpression( CONSTCHAR Expression ) { ErrorSetExpression_(uCalcHandle,Expression);       }
      
      /// <summary> Parses and evaluates an expression in one step</summary>
      /// <param name="Expression"> Expression to be evaluated</param>
      /// <remarks><para>This takes a math expression and returns a numeric result.</para></remarks> 
      double Eval( CONSTSTRING Expression ) { return  Eval_(uCalcHandle,CNV(Expression));       }
      
      double Eval( CONSTCHAR Expression ) { return  Eval_(uCalcHandle,Expression);       }
      
      /// <summary> Parses and evaluates an expression of any data type in one step, returning the result as a string</summary>
      /// <param name="Expression"> Expression to be evaluated</param>
      /// <remarks><para>This is similar to Eval, however, it can return non-numeric results, such as strings.</para></remarks> 
      STRING EvalStr( CONSTSTRING Expression ) { RETURNSTRING(EvalStr_(uCalcHandle,CNV(Expression)));       }
      
      STRING EvalStr( CONSTCHAR Expression ) { RETURNSTRING(EvalStr_(uCalcHandle,Expression));       }
      
      /// <summary> Clears floating point exception(s) that were raised</summary>
      /// <param name="Exceptions"> Exceptions</param>
      int32_t FPEnvClearException(  int32_t Exceptions  = 0) { return  FPEnvClearException_(uCalcHandle,Exceptions);       }
      
      /// <summary> Returns the floating point environment error mask</summary>
      int32_t FPEnvGetExceptionMask() { return  FPEnvGetExceptionMask_(uCalcHandle);       }
      
      /// <summary> Raises a floating point exception in a uCalc.Callback function</summary>
      /// <param name="Exception"> Bitmask value consisting of one or more exception flags to raise</param>
      /// <returns> Returns a 0 if all the specified execptions were raised successfully, otherwise a non-zero value is returned.</returns>
      /// <remarks><para>This attempts to raise the exception(s) specfied by the Exception parameter.  This is based on the C++
      /// feraiseexcept() function.  If you are using C++, you may use the pre-defined macro values of FE_DIVBYZERO,
      /// FE_INEXACT, FE_INVALID, FE_OVERFLOW, FE_UNDERFLOW, FE_ALL_EXCEPT.  If you are using another compiler, you may
      /// use the matching members of the FPEnvExceptionEnum enumerator.
      /// </para></remarks>
      int32_t FPEnvRaiseException(  FPEnvExceptionEnum Exception  = Invalid) { return  FPEnvRaiseException_(uCalcHandle,Exception);       }
      
      /// <summary> Gets the current floating point environment rounding option</summary>
      FPEnvRoundingEnum FPEnvGetRounding() { return  FPEnvGetRounding_(uCalcHandle);       }
      
      /// <summary> Sets the current floating point environment mask</summary>
      /// <param name="Exceptions"> Exceptions (FPEnvExceptionEnum)</param>
      /// <remarks><para>
      /// The floating point environment can be set to a value from FPEnvExceptionEnum.  If you are setting multiple values, then
      /// add them up.  For instance: FPEnvSetExceptionMask(Inexact + Overflow + Invalid);
      /// </para></remarks>
      void FPEnvSetExceptionMask( int32_t Exceptions) { FPEnvSetExceptionMask_(uCalcHandle,Exceptions);       }
      
      /// <summary> Unmasks the current floating point environment mask</summary>
      /// <param name="Exceptions"> Exceptions (FPEnvExceptionEnum)</param>
      /// <remarks><para>
      /// This unmasks (turns off) the floating point exceptions for the values are passed.  If you are setting multiple values, then
      /// add them up.  For instance: FPEnvSetExceptionUnMask(Inexact + Overflow + Invalid);
      /// </para></remarks>
      void FPEnvSetExceptionUnMask( int32_t Exceptions) { FPEnvSetExceptionUnMask_(uCalcHandle,Exceptions);       }
      
      /// <summary> Sets the floating point environment rounding option</summary>
      /// <param name="Direction"> Direction for rouding</param>
      /// <remarks><para>
      /// This sets the rounding mode for floating points.
      /// </para></remarks>
      int32_t FPEnvSetRounding( FPEnvRoundingEnum Direction) { return  FPEnvSetRounding_(uCalcHandle,Direction);       }
      
      /// <summary> Toggles the floating point environment mask</summary>
      /// <param name="Exception"> Exception to toggle</param>
      /// <remarks><para>
      /// This toggles an individual exception.
      /// </para></remarks> 
      void FPEnvToggleExceptionMask( FPEnvExceptionEnum Exception) { FPEnvToggleExceptionMask_(uCalcHandle,Exception);       }
      
      /// <summary> Returns the handle of a uCalc.Item given a name</summary>
      /// <param name="ItemName"> Name of the item who&#39;s handle you want</param>
      /// <param name="Properties"> Properties to narrow down the search to the desired item, where the same name is shared by multiple items with different properties</param>
      /// <param name="SkipTransform"> Skips any transform that would normally be associated with the item name, before returning a value (for instance, if you are interested in the transform itself)</param>
      uCalcPtr HandleOf( CONSTSTRING ItemName  ,  int64_t Properties  = 0,  bool SkipTransform = false) { return  HandleOf_(uCalcHandle,CNV(ItemName) ,Properties,SkipTransform);       }
      
      uCalcPtr HandleOf( CONSTCHAR ItemName  ,  int64_t Properties  = 0,  bool SkipTransform = false) { return  HandleOf_(uCalcHandle,ItemName ,Properties,SkipTransform);       }
      
      /// <summary> Returns the uCalc.Item object associated with the given name</summary>
      /// <param name="ItemName"> Name of uCalc.Item that you want to get</param>
      /// <param name="Properties"> Properties to help pinpoint the right Item, in case several items share the same name</param>
      /// <param name="SkipExpansion"> Some items are define as a syntax construct alias of another item.  By default the symbol is expanded first.  However, use this option if the alias is the object you&#39;re actually after.</param>
      /// <remarks><para>
      /// If there is no match, an item whose IsProperty ItemIsEnum.NotFound value is true is returned.
      /// </para><para>
      /// In case multiple items share the same name, you can narrow it down using properties.  For instance, the + operator has a
      /// binary and unary versions.  You can specify Prefix if you wanted the unary one.
      /// </para><para>
      /// Note: A ItemIsEnum member should not be passed directly as the second argument.  Instead Properties, with one or more
      /// ItemIsEnum members should be passed.
      /// </para></remarks> 
      Item GetItemOf( CONSTSTRING ItemName  ,  int64_t Properties  = 0,  bool SkipExpansion = false) { Item TempObj(GetItemOf_(uCalcHandle,CNV(ItemName) ,Properties,SkipExpansion)); return TempObj;       }
      
      Item GetItemOf( CONSTCHAR ItemName  ,  int64_t Properties  = 0,  bool SkipExpansion = false) { Item TempObj(GetItemOf_(uCalcHandle,ItemName ,Properties,SkipExpansion)); return TempObj;       }
      
      /// <summary> Returns the nth uCalc.Item object that matches a property</summary>
      /// <param name="Property_"> Property of the uCalc.Item object you&#39;re after</param>
      /// <param name="nth"> The nth object</param>
      /// <param name="DataTypeName"> The data type of the object</param>
      /// <remarks><para>
      /// This can be used to cycle through defined items.
      /// If there is no match, an item whose IsProperty ItemIsEnum.NotFound value is true is returned.
      /// </para></remarks> 
      Item GetItemOfProperty( ItemIsEnum Property_ ,  int32_t nth ,  CONSTSTRING DataTypeName  = EMPTYSTR) { Item TempObj(GetItemOfProperty_(uCalcHandle,Property_ , nth ,CNV(DataTypeName))); return TempObj;       }
      
      Item GetItemOfProperty( ItemIsEnum Property_ ,  int32_t nth ,  CONSTCHAR DataTypeName  = "") { Item TempObj(GetItemOfProperty_(uCalcHandle,Property_ , nth ,DataTypeName)); return TempObj;       }
      
      /// <summary> Sets the license code</summary>
      int32_t License( LicenseTypeEnum LicenseType,   int32_t L1 = 0,   int32_t L2 = 0,   int32_t L3 = 0,   int32_t L4 = 0,   int32_t L5 = 0) { return  License_(uCalcHandle,LicenseType, L1, L2, L3, L4, L5);       }
      
      /// <summary> ???</summary>
      Item NamespaceChild(  int32_t nth = 0) { Item TempObj(NamespaceChild_(uCalcHandle,nth)); return TempObj;       }
      
      /// <summary> ???</summary>
      int32_t NamespaceChildCount() { return  NamespaceChildCount_(uCalcHandle);       }
      
      /// <summary> ???</summary>
      Item NamespaceParent(  int32_t Level = 0) { Item TempObj(NamespaceParent_(uCalcHandle,Level)); return TempObj;       }
      
      /// <summary> ???</summary>
      Item NamespaceSymbol( int32_t nth) { Item TempObj(NamespaceSymbol_(uCalcHandle,nth)); return TempObj;       }
      
      /// <summary> ???</summary>
      int32_t NamespaceSymbolCount() { return  NamespaceSymbolCount_(uCalcHandle);       }
      
      /// <summary> Returns a new transform object</summary>
      /// <remarks><para>
      /// Returns a new object that can be used to transform text.
      /// </para></remarks>
      uCalcTransform NewTransform() { uCalcTransform TempObj(NewTransform_(uCalcHandle)); return TempObj;       }
      
      /// <summary> Returns the transform object associated with the current math parser object</summary>
      /// <remarks><para>
      /// Whenever a math expression is parsed, this transform is first called.  Use this when you want transforms that apply to math
      /// expressions parsed with Parse, Eval, or EvalStr.  To create a new transform that does not affect math expressions sent to
      /// Parse, use NewTransform instead.
      /// </para><para>
      /// Independent transforms created with NewTransform can be released when no longer needed.  However, the object created with
      /// ParserTransform should never be released explicitely, because the math parser object requires it in order to function properly.
      /// </para></remarks> 
      uCalcTransform ParserTransform() { uCalcTransform TempObj(ParserTransform_(uCalcHandle)); return TempObj;       }
      
      /// <summary> Parses an expression that will later be evaluated with Evaluate or EvaluateStr</summary>
      /// <param name="Expression"> Expression to be parsed</param>
      /// <param name="DataType"> Data type of return value</param>
      ParsedExpr Parse( CONSTSTRING Expression  ,  CONSTSTRING DataType  = EMPTYSTR) { ParsedExpr TempObj(Parse_(uCalcHandle,CNV(Expression) ,CNV(DataType))); return TempObj;       }
      
      ParsedExpr Parse( CONSTCHAR Expression  ,  CONSTCHAR DataType  = "") { ParsedExpr TempObj(Parse_(uCalcHandle,Expression ,DataType)); return TempObj;       }
      
      /// <summary> Returns the precedence level of an operator or the level between two operators</summary>
      /// <param name="Operator1"> Operator with lower (or equal) precedence</param>
      /// <param name="Operator2"> Operator with greater (or equal precedence)</param>
      /// <remarks><para>
      /// This returns a precedence level based on levels of existing operators; useful when defining a new operator (or transform)
      /// with a level relative to levels of existing operators (or transforms).  If one argument is passed, then the precedence
      /// level of that item is returned.  This is useful for defining a new item at the same level as the specified item.  If two
      /// arguments are passed, then the average precedence level of the two items is returned.  This is useful for defining a level
      /// that&#39;s in between that of two existing items.
      /// </para></remarks>
      int32_t  Precedence( CONSTSTRING Operator1  ,  CONSTSTRING Operator2  = EMPTYSTR) { return  Precedence_Overload_uCalc_(uCalcHandle,CNV(Operator1) ,CNV(Operator2));       }
      
      int32_t  Precedence( CONSTCHAR Operator1  ,  CONSTCHAR Operator2  = "") { return  Precedence_Overload_uCalc_(uCalcHandle,Operator1 ,Operator2);       }
      
      /// <summary> Combines properties into a numeric value that can be passed to GetItemOf</summary>
      /// <remarks><para>
      /// ItemIsEnum properties are numbered sequentially (0, 1, 2, 3, 4, ...).  However the Properties argument
      /// of the GetItemOf function requires properties in the 2^x sequence (1, 2, 4, 8, 16, ...).  So you
      /// must pass one or more ItemIsEnum to Properties before passing it to GetItemOf.
      /// </para></remarks>
      int64_t Properties( ItemIsEnum p1,   ItemIsEnum p2 = AnyItem,   ItemIsEnum p3 = AnyItem,   ItemIsEnum p4 = AnyItem) { return  Properties_(uCalcHandle,p1, p2, p3, p4);       }
      
      /// <summary> (Use Release instead)</summary>
      /// <remarks><para>Use Release</para></remarks> 
      void ReleaseuCalc() { ReleaseuCalc_(uCalcHandle);       }
      
      /// <summary> Removes any output decoration</summary>
      void ResetOutput() { ResetOutput_(uCalcHandle);       }
      
      /// <summary> Returns default settings</summary>
      /// <remarks><para>The Item returned contains the default settings for the current uCalc instance.  To define new settings,
      /// use Define(&quot;Default ~~ ...&quot;).</para></remarks> 
      Item Default() { Item TempObj(Default_(uCalcHandle)); return TempObj;       }
      
      /// <summary> Allows a callback to decorate the output of EvalStr</summary>
      /// <param name="FunctionAddress"> Address of callback function that formats the output</param>
      /// <param name="Properties"> Name of datatype whose output will be formated and/or ranking among other output formats; leave blank to affect all data types and use default ranking.</param>
      Item  SetOutput( UCCALLBACK FunctionAddress ,  CONSTSTRING Properties ) { Item TempObj(SetOutput_Overload_uCalc__Callback_(uCalcHandle,FunctionAddress , CNV(Properties))); return TempObj;       }
      
      Item  SetOutput( UCCALLBACK FunctionAddress ,  CONSTCHAR Properties ) { Item TempObj(SetOutput_Overload_uCalc__Callback_(uCalcHandle,FunctionAddress , Properties)); return TempObj;       }
      
      /// <summary> Formats the appearance of results returned by EvalStr or EvaluateStr</summary>
      /// <param name="Definition"> Format definition for the output</param>
      /// <remarks><para>
      /// This function lets you modify the output returned by EvalStr or EvaluateStr.  The definition may contain one or more
      /// of the following:
      /// </para><para>
      /// Datatype:  If you specify a data type, then the modification will affect only the evaluation result of expressions
      /// that evaluate to the specified data type.
      /// </para><para>
      /// InsertAt:  By default, if you define multiple outputs, then the one defined most recently is applied first, followed by
      /// the other outputs in reverse order in which they were defined, ending with the output that was defined first.  You may
      /// specify a different insertion point than the default with this parameter.
      /// </para><para>
      /// Expression:  This represents what the modified output should look like.  The expression consists of an identifier
      /// (any temporary variable name), followed by an equal sign, followed by a string expression representing the value
      /// that will be returned.  The variable holds the original value that would have been returned.
      /// </para></remarks>
      Item  SetOutput( CONSTSTRING Definition ) { Item TempObj(SetOutput_Overload_uCalc__NoCallback_(uCalcHandle,CNV(Definition))); return TempObj;       }
      
      Item  SetOutput( CONSTCHAR Definition ) { Item TempObj(SetOutput_Overload_uCalc__NoCallback_(uCalcHandle,Definition)); return TempObj;       }
      
      /// <summary> Creates a transform that can modify text or search through text</summary>
      /// <returns> Returns a transform object</returns>
      /// <remarks><para>
      /// +++
      /// </para></remarks>
      Item CreateTransform() { Item TempObj(CreateTransform_(uCalcHandle)); return TempObj;       }
      
      /// <summary> Returns the value located at the specified Pointer address</summary>
      /// <param name="ValuePtr"> Pointer address containing the data</param>
      /// <param name="DataType"> Data type of the data you need, which helps to determine in which form to return the value at the given location</param>
      /// <remarks><para>+++EXP ORT STR_RETURN ValueAt(BYVAL uCalcPtr uCalcHandle, BYVAL POINTER ValuePtr, OBJ DataType)
      /// +++ SetValueAt
      /// </para></remarks>
      STRING ValueAt( void * ValuePtr ,  CONSTSTRING DataType ) { RETURNSTRING(ValueAt_(uCalcHandle,ValuePtr , CNV(DataType)));       }
      
      STRING ValueAt( void * ValuePtr ,  CONSTCHAR DataType ) { RETURNSTRING(ValueAt_(uCalcHandle,ValuePtr , DataType));       }
      
      /// <summary> Returns the handle of the uCalc object</summary>
      /// <remarks><para>Generally for internal use</para></remarks>
      void *  Handle() { return  Handle_Overload_uCalc_(uCalcHandle);       }
      
      void Release() { ReleaseuCalc(); uCalcHandle = 0;       }
      /// <summary> Releases a uCalc instance</summary>
      /// <remarks><para>When a uCalc instance is no longer needed, it can be released with this function.  This releases all items
      /// associated with the uCalc instance.
      /// </para><para>
      /// Note: The uCalc object named uc, which is instantiated at startup (except in C#) should not be explicitely released.
      /// In C#, if there is only 1 uCalc object left, do not explicitely release it.
      /// </para><para>
      /// Note: A uCalc object must explicitely be released with Release for it to be released.  A uCalc object is not
      /// automatically released from memory when the object goes out of scope.</para></remarks>
      
      void Reset() { ReleaseuCalc(); uCalcHandle = CreateuCalc();       }
      uCalc() {
         static HINSTANCE hLib = NULL;
         if (hLib == NULL) {
            hLib = LoadLibraryA(uCalcLib);
            Evaluate_ = (__Evaluate)GetProcAddress(hLib, "Evaluate");
            EvaluateNTE_ = (__EvaluateNTE)GetProcAddress(hLib, "EvaluateNTE");
            EvaluateInt32_ = (__EvaluateInt32)GetProcAddress(hLib, "EvaluateInt32");
            EvaluateStr_ = (__EvaluateStr)GetProcAddress(hLib, "EvaluateStr");
            EvaluateVoid_ = (__EvaluateVoid)GetProcAddress(hLib, "EvaluateVoid");
            Execute_ = (__Execute)GetProcAddress(hLib, "Execute");
            ReleaseExpr_ = (__ReleaseExpr)GetProcAddress(hLib, "ReleaseExpr");
            Handle_Overload_ParsedExpr_ = (__Handle_Overload_ParsedExpr)GetProcAddress(hLib, "Handle_Overload_ParsedExpr");
            ElementCount_ = (__ElementCount)GetProcAddress(hLib, "ElementCount");
            GetDataType_ = (__GetDataType)GetProcAddress(hLib, "GetDataType");
            FunctionAddress_ = (__FunctionAddress)GetProcAddress(hLib, "FunctionAddress");
            IsProperty_ = (__IsProperty)GetProcAddress(hLib, "IsProperty");
            Name_Overload_Item_ = (__Name_Overload_Item)GetProcAddress(hLib, "Name_Overload_Item");
            NextOverload_ = (__NextOverload)GetProcAddress(hLib, "NextOverload");
            Precedence_Overload_Item_ = (__Precedence_Overload_Item)GetProcAddress(hLib, "Precedence_Overload_Item");
            ReleaseItem_ = (__ReleaseItem)GetProcAddress(hLib, "ReleaseItem");
            Rename_ = (__Rename)GetProcAddress(hLib, "Rename");
            SetVariableValue_ = (__SetVariableValue)GetProcAddress(hLib, "SetVariableValue");
            SetVariableValueAny_ = (__SetVariableValueAny)GetProcAddress(hLib, "SetVariableValueAny");
            SetVariableValueByte_ = (__SetVariableValueByte)GetProcAddress(hLib, "SetVariableValueByte");
            SetVariableValueInt16_ = (__SetVariableValueInt16)GetProcAddress(hLib, "SetVariableValueInt16");
            SetVariableValueInt32_ = (__SetVariableValueInt32)GetProcAddress(hLib, "SetVariableValueInt32");
            SetVariableValueInt64_ = (__SetVariableValueInt64)GetProcAddress(hLib, "SetVariableValueInt64");
            SetVariableValueSng_ = (__SetVariableValueSng)GetProcAddress(hLib, "SetVariableValueSng");
            SetVariableValueDbl_ = (__SetVariableValueDbl)GetProcAddress(hLib, "SetVariableValueDbl");
            SetVariableValueStr_ = (__SetVariableValueStr)GetProcAddress(hLib, "SetVariableValueStr");
            SetVariableValuePtr_ = (__SetVariableValuePtr)GetProcAddress(hLib, "SetVariableValuePtr");
            SetVariableValueByPtr_ = (__SetVariableValueByPtr)GetProcAddress(hLib, "SetVariableValueByPtr");
            Text_ = (__Text)GetProcAddress(hLib, "Text");
            GetuCalc_Overload_Item_ = (__GetuCalc_Overload_Item)GetProcAddress(hLib, "GetuCalc_Overload_Item");
            Value_ = (__Value)GetProcAddress(hLib, "Value");
            ValueByte_ = (__ValueByte)GetProcAddress(hLib, "ValueByte");
            ValueInt16_ = (__ValueInt16)GetProcAddress(hLib, "ValueInt16");
            ValueInt32_ = (__ValueInt32)GetProcAddress(hLib, "ValueInt32");
            ValueInt64_ = (__ValueInt64)GetProcAddress(hLib, "ValueInt64");
            ValueSng_ = (__ValueSng)GetProcAddress(hLib, "ValueSng");
            ValueDbl_ = (__ValueDbl)GetProcAddress(hLib, "ValueDbl");
            ValueStr_ = (__ValueStr)GetProcAddress(hLib, "ValueStr");
            ValuePtr_ = (__ValuePtr)GetProcAddress(hLib, "ValuePtr");
            ValueAddr_ = (__ValueAddr)GetProcAddress(hLib, "ValueAddr");
            Handle_Overload_Item_ = (__Handle_Overload_Item)GetProcAddress(hLib, "Handle_Overload_Item");
            Arg_ = (__Arg)GetProcAddress(hLib, "Arg");
            ArgCount_ = (__ArgCount)GetProcAddress(hLib, "ArgCount");
            ArgObj_ = (__ArgObj)GetProcAddress(hLib, "ArgObj");
            ArgAddr_ = (__ArgAddr)GetProcAddress(hLib, "ArgAddr");
            ArgExpr_ = (__ArgExpr)GetProcAddress(hLib, "ArgExpr");
            ArgInt32_ = (__ArgInt32)GetProcAddress(hLib, "ArgInt32");
            ArgInt64_ = (__ArgInt64)GetProcAddress(hLib, "ArgInt64");
            ArgPtr_ = (__ArgPtr)GetProcAddress(hLib, "ArgPtr");
            ArgStr_ = (__ArgStr)GetProcAddress(hLib, "ArgStr");
            Return_ = (__Return)GetProcAddress(hLib, "Return");
            ReturnInt32_ = (__ReturnInt32)GetProcAddress(hLib, "ReturnInt32");
            ReturnInt64_ = (__ReturnInt64)GetProcAddress(hLib, "ReturnInt64");
            ReturnPtr_ = (__ReturnPtr)GetProcAddress(hLib, "ReturnPtr");
            ReturnStr_ = (__ReturnStr)GetProcAddress(hLib, "ReturnStr");
            ErrorRaise_ = (__ErrorRaise)GetProcAddress(hLib, "ErrorRaise");
            ErrorRaiseMessage_ = (__ErrorRaiseMessage)GetProcAddress(hLib, "ErrorRaiseMessage");
            GetuCalc_Overload_Callback_ = (__GetuCalc_Overload_Callback)GetProcAddress(hLib, "GetuCalc_Overload_Callback");
            Handle_Overload_Callback_ = (__Handle_Overload_Callback)GetProcAddress(hLib, "Handle_Overload_Callback");
            OpItem_ = (__OpItem)GetProcAddress(hLib, "OpItem");
            AllocArray_ = (__AllocArray)GetProcAddress(hLib, "AllocArray");
            AllocByPtr_ = (__AllocByPtr)GetProcAddress(hLib, "AllocByPtr");
            AllocScalar_ = (__AllocScalar)GetProcAddress(hLib, "AllocScalar");
            ByteSize_ = (__ByteSize)GetProcAddress(hLib, "ByteSize");
            Index_ = (__Index)GetProcAddress(hLib, "Index");
            GetItem_ = (__GetItem)GetProcAddress(hLib, "GetItem");
            Name_Overload_DataType_ = (__Name_Overload_DataType)GetProcAddress(hLib, "Name_Overload_DataType");
            ReleaseArray_ = (__ReleaseArray)GetProcAddress(hLib, "ReleaseArray");
            ReleaseScalar_ = (__ReleaseScalar)GetProcAddress(hLib, "ReleaseScalar");
            ResetArray_ = (__ResetArray)GetProcAddress(hLib, "ResetArray");
            ResetScalar_ = (__ResetScalar)GetProcAddress(hLib, "ResetScalar");
            SetArray_ = (__SetArray)GetProcAddress(hLib, "SetArray");
            SetScalar_ = (__SetScalar)GetProcAddress(hLib, "SetScalar");
            SwapScalarValues_ = (__SwapScalarValues)GetProcAddress(hLib, "SwapScalarValues");
            ToString_Overload_DataType__ByPtr_ = (__ToString_Overload_DataType__ByPtr)GetProcAddress(hLib, "ToString_Overload_DataType__ByPtr");
            ToString_Overload_DataType__ByStr_ = (__ToString_Overload_DataType__ByStr)GetProcAddress(hLib, "ToString_Overload_DataType__ByStr");
            Handle_Overload_DataType_ = (__Handle_Overload_DataType)GetProcAddress(hLib, "Handle_Overload_DataType");
            Active_ = (__Active)GetProcAddress(hLib, "Active");
            FindPart_ = (__FindPart)GetProcAddress(hLib, "FindPart");
            FromPart_ = (__FromPart)GetProcAddress(hLib, "FromPart");
            ToPart_ = (__ToPart)GetProcAddress(hLib, "ToPart");
            CaseSensitive_ = (__CaseSensitive)GetProcAddress(hLib, "CaseSensitive");
            WhitespaceSensitive_ = (__WhitespaceSensitive)GetProcAddress(hLib, "WhitespaceSensitive");
            CodeBlockSensitive_ = (__CodeBlockSensitive)GetProcAddress(hLib, "CodeBlockSensitive");
            Comment_Overload_uCalcTransform_RuleClass_ = (__Comment_Overload_uCalcTransform_RuleClass)GetProcAddress(hLib, "Comment_Overload_uCalcTransform_RuleClass");
            LocalTransform_ = (__LocalTransform)GetProcAddress(hLib, "LocalTransform");
            Maximum_ = (__Maximum)GetProcAddress(hLib, "Maximum");
            MaximumHard_ = (__MaximumHard)GetProcAddress(hLib, "MaximumHard");
            Minimum_ = (__Minimum)GetProcAddress(hLib, "Minimum");
            MinimumHard_ = (__MinimumHard)GetProcAddress(hLib, "MinimumHard");
            Precedence_ = (__Precedence)GetProcAddress(hLib, "Precedence");
            QuoteSensitive_ = (__QuoteSensitive)GetProcAddress(hLib, "QuoteSensitive");
            ReplacementText_ = (__ReplacementText)GetProcAddress(hLib, "ReplacementText");
            StartAfter_ = (__StartAfter)GetProcAddress(hLib, "StartAfter");
            StopAfter_ = (__StopAfter)GetProcAddress(hLib, "StopAfter");
            SkipOver_ = (__SkipOver)GetProcAddress(hLib, "SkipOver");
            RewindOnChange_ = (__RewindOnChange)GetProcAddress(hLib, "RewindOnChange");
            ParseAgainMax_ = (__ParseAgainMax)GetProcAddress(hLib, "ParseAgainMax");
            RightToLeft_ = (__RightToLeft)GetProcAddress(hLib, "RightToLeft");
            ReleaseRule_ = (__ReleaseRule)GetProcAddress(hLib, "ReleaseRule");
            Comment_Overload_uCalcTransform_Tokens_ = (__Comment_Overload_uCalcTransform_Tokens)GetProcAddress(hLib, "Comment_Overload_uCalcTransform_Tokens");
            Import_Overload_uCalcTransform_Tokens__Obj_ = (__Import_Overload_uCalcTransform_Tokens__Obj)GetProcAddress(hLib, "Import_Overload_uCalcTransform_Tokens__Obj");
            Import_Overload_uCalcTransform_Tokens__File_ = (__Import_Overload_uCalcTransform_Tokens__File)GetProcAddress(hLib, "Import_Overload_uCalcTransform_Tokens__File");
            Define_Overload_uCalcTransform_Tokens_ = (__Define_Overload_uCalcTransform_Tokens)GetProcAddress(hLib, "Define_Overload_uCalcTransform_Tokens");
            Comment_Overload_uCalcTransform_ = (__Comment_Overload_uCalcTransform)GetProcAddress(hLib, "Comment_Overload_uCalcTransform");
            DefineRule_ = (__DefineRule)GetProcAddress(hLib, "DefineRule");
            DefineToken_ = (__DefineToken)GetProcAddress(hLib, "DefineToken");
            GetTokens_ = (__GetTokens)GetProcAddress(hLib, "GetTokens");
            FromTo_ = (__FromTo)GetProcAddress(hLib, "FromTo");
            Import_Overload_uCalcTransform_ = (__Import_Overload_uCalcTransform)GetProcAddress(hLib, "Import_Overload_uCalcTransform");
            ListPrefix_ = (__ListPrefix)GetProcAddress(hLib, "ListPrefix");
            ListPostfix_ = (__ListPostfix)GetProcAddress(hLib, "ListPostfix");
            ListSeparator_ = (__ListSeparator)GetProcAddress(hLib, "ListSeparator");
            ListDelimiters_ = (__ListDelimiters)GetProcAddress(hLib, "ListDelimiters");
            MathParser_ = (__MathParser)GetProcAddress(hLib, "MathParser");
            NewPass_ = (__NewPass)GetProcAddress(hLib, "NewPass");
            Pass_ = (__Pass)GetProcAddress(hLib, "Pass");
            Reset_ = (__Reset)GetProcAddress(hLib, "Reset");
            ResultStr_ = (__ResultStr)GetProcAddress(hLib, "ResultStr");
            Transform_ = (__Transform)GetProcAddress(hLib, "Transform");
            Expand_ = (__Expand)GetProcAddress(hLib, "Expand");
            ExpandWithSteps_ = (__ExpandWithSteps)GetProcAddress(hLib, "ExpandWithSteps");
            SetInput_ = (__SetInput)GetProcAddress(hLib, "SetInput");
            SetInputByRef_ = (__SetInputByRef)GetProcAddress(hLib, "SetInputByRef");
            SetParser_ = (__SetParser)GetProcAddress(hLib, "SetParser");
            SetProperties_ = (__SetProperties)GetProcAddress(hLib, "SetProperties");
            WasModified_ = (__WasModified)GetProcAddress(hLib, "WasModified");
            ReleaseTransform_ = (__ReleaseTransform)GetProcAddress(hLib, "ReleaseTransform");
            GetRule_ = (__GetRule)GetProcAddress(hLib, "GetRule");
            RulesDefault_ = (__RulesDefault)GetProcAddress(hLib, "RulesDefault");
            RulesOverall_ = (__RulesOverall)GetProcAddress(hLib, "RulesOverall");
            AddErrorHandler_ = (__AddErrorHandler)GetProcAddress(hLib, "AddErrorHandler");
            CreateuCalc_ = (__CreateuCalc)GetProcAddress(hLib, "CreateuCalc");
            DataTypeOfIndex_ = (__DataTypeOfIndex)GetProcAddress(hLib, "DataTypeOfIndex");
            Define_Overload_uCalc_ = (__Define_Overload_uCalc)GetProcAddress(hLib, "Define_Overload_uCalc");
            DefineConstant_ = (__DefineConstant)GetProcAddress(hLib, "DefineConstant");
            DefineDataType_ = (__DefineDataType)GetProcAddress(hLib, "DefineDataType");
            DefineFunction_ = (__DefineFunction)GetProcAddress(hLib, "DefineFunction");
            DefineOperator_ = (__DefineOperator)GetProcAddress(hLib, "DefineOperator");
            DefineVariable_ = (__DefineVariable)GetProcAddress(hLib, "DefineVariable");
            ErrorExpression_ = (__ErrorExpression)GetProcAddress(hLib, "ErrorExpression");
            ErrorLocation_ = (__ErrorLocation)GetProcAddress(hLib, "ErrorLocation");
            ErrorMessage_ = (__ErrorMessage)GetProcAddress(hLib, "ErrorMessage");
            ErrorNumber_ = (__ErrorNumber)GetProcAddress(hLib, "ErrorNumber");
            ErrorResponse_ = (__ErrorResponse)GetProcAddress(hLib, "ErrorResponse");
            ErrorSymbol_ = (__ErrorSymbol)GetProcAddress(hLib, "ErrorSymbol");
            ErrorSetExpression_ = (__ErrorSetExpression)GetProcAddress(hLib, "ErrorSetExpression");
            Eval_ = (__Eval)GetProcAddress(hLib, "Eval");
            EvalStr_ = (__EvalStr)GetProcAddress(hLib, "EvalStr");
            FPEnvClearException_ = (__FPEnvClearException)GetProcAddress(hLib, "FPEnvClearException");
            FPEnvGetExceptionMask_ = (__FPEnvGetExceptionMask)GetProcAddress(hLib, "FPEnvGetExceptionMask");
            FPEnvRaiseException_ = (__FPEnvRaiseException)GetProcAddress(hLib, "FPEnvRaiseException");
            FPEnvGetRounding_ = (__FPEnvGetRounding)GetProcAddress(hLib, "FPEnvGetRounding");
            FPEnvSetExceptionMask_ = (__FPEnvSetExceptionMask)GetProcAddress(hLib, "FPEnvSetExceptionMask");
            FPEnvSetExceptionUnMask_ = (__FPEnvSetExceptionUnMask)GetProcAddress(hLib, "FPEnvSetExceptionUnMask");
            FPEnvSetRounding_ = (__FPEnvSetRounding)GetProcAddress(hLib, "FPEnvSetRounding");
            FPEnvToggleExceptionMask_ = (__FPEnvToggleExceptionMask)GetProcAddress(hLib, "FPEnvToggleExceptionMask");
            HandleOf_ = (__HandleOf)GetProcAddress(hLib, "HandleOf");
            GetItemOf_ = (__GetItemOf)GetProcAddress(hLib, "GetItemOf");
            GetItemOfProperty_ = (__GetItemOfProperty)GetProcAddress(hLib, "GetItemOfProperty");
            License_ = (__License)GetProcAddress(hLib, "License");
            NamespaceChild_ = (__NamespaceChild)GetProcAddress(hLib, "NamespaceChild");
            NamespaceChildCount_ = (__NamespaceChildCount)GetProcAddress(hLib, "NamespaceChildCount");
            NamespaceParent_ = (__NamespaceParent)GetProcAddress(hLib, "NamespaceParent");
            NamespaceSymbol_ = (__NamespaceSymbol)GetProcAddress(hLib, "NamespaceSymbol");
            NamespaceSymbolCount_ = (__NamespaceSymbolCount)GetProcAddress(hLib, "NamespaceSymbolCount");
            NewTransform_ = (__NewTransform)GetProcAddress(hLib, "NewTransform");
            ParserTransform_ = (__ParserTransform)GetProcAddress(hLib, "ParserTransform");
            Parse_ = (__Parse)GetProcAddress(hLib, "Parse");
            Precedence_Overload_uCalc_ = (__Precedence_Overload_uCalc)GetProcAddress(hLib, "Precedence_Overload_uCalc");
            Properties_ = (__Properties)GetProcAddress(hLib, "Properties");
            ReleaseuCalc_ = (__ReleaseuCalc)GetProcAddress(hLib, "ReleaseuCalc");
            ResetOutput_ = (__ResetOutput)GetProcAddress(hLib, "ResetOutput");
            Default_ = (__Default)GetProcAddress(hLib, "Default");
            SetOutput_Overload_uCalc__Callback_ = (__SetOutput_Overload_uCalc__Callback)GetProcAddress(hLib, "SetOutput_Overload_uCalc__Callback");
            SetOutput_Overload_uCalc__NoCallback_ = (__SetOutput_Overload_uCalc__NoCallback)GetProcAddress(hLib, "SetOutput_Overload_uCalc__NoCallback");
            CreateTransform_ = (__CreateTransform)GetProcAddress(hLib, "CreateTransform");
            ValueAt_ = (__ValueAt)GetProcAddress(hLib, "ValueAt");
            Handle_Overload_uCalc_ = (__Handle_Overload_uCalc)GetProcAddress(hLib, "Handle_Overload_uCalc");
            
         }
         
         uCalcHandle = 0; uCalcHandle = CreateuCalc();
      }
      uCalc(const uCalc & uCalcInstance) { uCalcHandle = uCalcInstance.uCalcHandle;       }
      uCalc( uCalcPtr InstHandle) { uCalcHandle = InstHandle;       }
      uCalc( LicenseTypeEnum LicenseType,   int32_t L1 = 0,   int32_t L2 = 0,   int32_t L3 = 0,   int32_t L4 = 0,   int32_t L5 = 0) { License(LicenseType, L1, L2, L3, L4, L5); uCalcHandle = CreateuCalc();       }
   }; // uCalc
   
#if !defined SkipuCalcAutoDef
   extern uCalc uc;
#endif
   
} // End of uCalcNameSpace
using namespace uCalcNameSpace;
